"""Subclass of ClassDiagramPanePane, which is generated by wxFormBuilder."""

import wx
import wxx
import tran
import app
from ctx import theContext as context
import model
import gettext
from app.ui.pane import BasePane
from activity.models.ui import ui as ui
from  activity.models.ui import view as view
from activity.models.ui import dlg

_ = gettext.gettext

OP_INSERT_CLASS = 1000
OP_MOVE_CLASS = 1001
OP_CLICK_CLASS = 1002
OP_CLICK_ELEMENT = 1003
OP_CLICK_SEGMENT = 1005
OP_MOVE_ELEMENT = 1004
OP_MOVE_SEGMENT = 1006

NEW_CLASS_RECTSIZE = wx.Size(80, 20)

ID_PRINT = 1026
CLOSE_PANE_ID = wx.ID_HIGHEST + 100


# Implementing ClassDiagramPanePane
class DiagramPane(BasePane, ui.ClassDiagramPane):
    """Implements classsDiagramPane pane"""
    _addNoteId = wx.Window.NewControlId()
    _selectClassesId = wx.Window.NewControlId()

    def __init__(self, parent, obj):
        """Initialization"""
        super(DiagramPane, self).__init__(parent)
        self._object = obj  # ClassDiagram object
        self.parent = parent  # notebook object
        self._operation = None
        self._element = None  # context element
        self._create_menus()
        #self._create_toolbars()
        self._bind_events()
        self._set_accelerators()

        szP = self.parent.GetClientSize()
        szP.y -= self.parent.GetTabCtrlHeight()
        sz = wx.Size(self._object._width, self._object._height)
        self.SetSize(szP)
        self.DoSetVirtualSize(sz.x, sz.y)
        self.SetScrollbars(5, 5, sz.x // 5, sz.y // 5, 0, 0, True)
        self._inPopup = False
        self.SetFocus()
        self.EnableScrolling(True, True)
        self.selected = None

    def Refresh(self):
        """Update editor from external changes (like undo/redo)"""
        self.Freeze()
        dc = self.AdjustedDC()
        dc.Clear()
        self._object.DrawClassDiagram(dc)
        self.Thaw()
        self.Update()

    def _create_menus(self):
        """Create a custom menu entry in main menu"""
        self._menu = wx.Menu()

        # Select classes
        self._select_classes_menu = wx.MenuItem(self._menu, self._selectClassesId, u"Select classes",
            u"select classes to be shown in this diagram", wx.ITEM_NORMAL)
        #self._newRepo.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_NEW, wx.ART_MENU))
        self._menu.AppendItem(self._select_classes_menu)

        #Add note
        self._add_note_menu = wx.MenuItem(self._menu, self._addNoteId, u"Add note",
            u"add text note tho this diagram", wx.ITEM_NORMAL)
        #self._openRepo.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_FILE_OPEN, wx.ART_MENU))
        self._menu.AppendItem(self._add_note_menu)

    def _bind_events(self):
        """Bind events to this frame"""
        self.BindSpecial(wx.EVT_MENU, self.OnSelectClasses, id=self._selectClassesId)
        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateSelectClasses, id=self._selectClassesId)

        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateDiagramAddNote, id=self._addNoteId)
        self.BindSpecial(wx.EVT_MENU, self.OnDiagramAddNote, id=self._addNoteId)

        frame = context.frame

        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateEditProperties, id=app.ID_EDIT_PROPERTIES)
        self.BindSpecial(wx.EVT_MENU, self.OnEditProperties, id=app.ID_EDIT_PROPERTIES)

        # add class
        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateAddClass, id=view.ModelsView._addClassId)
        self.BindSpecial(wx.EVT_MENU, self.OnAddClass, id=view.ModelsView._addClassId)
        # save diagram
        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateSave, id=app.ID_SAVE_PROJECT)
        self.BindSpecial(wx.EVT_MENU, self.OnSave, id=app.ID_SAVE_PROJECT)
        # delete element
        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateDelete, id=app.ID_DELETE)
        self.BindSpecial(wx.EVT_MENU, self.OnDelete, id=app.ID_DELETE)
        # close pane
        self.BindSpecial(wx.EVT_MENU, frame.CloseCurrentDocPane, id=CLOSE_PANE_ID)
        # add relations
        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateAddRelation, id=view.ModelsView._addRelationId)
        self.BindSpecial(wx.EVT_MENU, self.OnAddRelation, id=view.ModelsView._addRelationId)

    def _set_accelerators(self):
        """Set the acceleratos"""
        frame = context.frame
        aTable = wx.AcceleratorTable([
            wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_DELETE, frame.delete.GetId()),
            wx.AcceleratorEntry(wx.ACCEL_ALT | wx.ACCEL_CTRL, ord('C'), view.ModelsView._addClassId),
            wx.AcceleratorEntry(wx.ACCEL_ALT | wx.ACCEL_CTRL, ord('F'), ui.ID_TOGGLE_FULLSCREEN),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, ord('W'), CLOSE_PANE_ID)
        ])
        self.SetAcceleratorTable(aTable)

    def NotifyShow(self):
        """Called where the view is show"""
        context.frame.AddMainMenu('Diagram', self._menu)
        super(DiagramPane, self).NotifyShow()

    def OnGetFocus(self, event):
        """Called where the view get focus"""
        super(DiagramPane, self).NotifyShow()

    def NotifyHide(self):
        """Called where the git view is hidden"""
        context.frame.RemoveMainMenu('Diagram', self._menu)
        super(DiagramPane, self).NotifyHide()

    def OnLeaveFocus(self, event):
        """Called where the view get focus"""
        super(DiagramPane, self).NotifyHide()

    @tran.TransactionalMethod('delete elements')
    def OnDelete(self, event):
        """Handle delete event"""
        dead = [x for x in self._object._elements if x.Selected()]
        for element in dead:
            element._obj.Delete()
        return True

    def OnUpdateDelete(self, event):
        """Handle update delete event"""
        for element in self._object._elements:
            if element.Selected():
                event.Enable(True)
                return
        event.Enable(False)

    @tran.TransactionalMethod('modify diagram')
    def OnSave(self, event):
        """Handle save event"""
        self.Commit()
        return True

    def OnUpdateSave(self, event):
        """Handle update save event"""
        event.Enable(False)
        return True

    def Commit(self):
        """Update document"""
        pass

    # Handlers for ClassDiagramPanePane events.
    def OnPaintClassDiagram(self, event):
        """Repaint class DiagramPane"""
        dc = wx.PaintDC(self)
        self.PrepareDC(dc)
        self._object.DrawClassDiagram(dc)

    def OnDiagramMenu(self, event):
        """Do DiagramPane menu"""
        if not type(self._object) is model.ClassDiagram:
            return
        self.SetFocus()
        wx.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))
        position = self.GetLocalPosition(False)
        (item, where) = self._object.HitTest(position)
        if type(item) is tuple:
            # se ha hecho click en un segmento
            item = item[0]
            menu = wx.Menu()
            if type(item._obj) is model.cc.Relation:
                self.selected = item._obj._FROM
            else:
                self.selected = item._obj
            menu.Append(context.frame.editProperties.GetId(),
                "Edit properties")
        elif item is None:
            menu = app.ui.clone_mnu(self._menu, enabled=True, notitle=True)
            if menu is None:
                return
        elif where is model.IN_LINE_MARK:
            menu = wx.Menu()
            self.selected = item._obj
            menu.Append(context.frame.editProperties.GetId(),
                "Edit properties")
        elif where is model.IN_SHAPE:
            if type(item) is model.ClassElement:
                menu = wx.Menu()
                self.selected = item._obj
                menu.Append(view.ModelsView._addRelationId, "Add relation")
                menu.Append(context.frame.editProperties.GetId(),
                    "Edit properties")
            elif type(item) is model.NoteElement:
                menu = wx.Menu()
                self.selected = item._obj
                menu.Append(context.frame.editProperties.GetId(),
                    "Edit properties")
        else:
            return
        self._inPopup = True
        self.PopupMenu(menu, event.GetPosition())
        menu.Destroy()

    def AdjustedDC(self):
        """Prepare DC for drawing"""
        dc = wx.ClientDC(self)
        self.PrepareDC(dc)
        return dc
        pen = wx.BLACK_PEN
        brush = wx.Brush(wx.TheColourDatabase.Find('AQUAMARINE'))
        dc.SetPen(pen)
        pen.SetWidth(2)
        dc.SetBrush(brush)
        dc.SetLogicalFunction(wx.COPY)
        return dc

    def GetLocalPosition(self, normalize=True, pos=wx.DefaultPosition):
        """Get the local position normalized to grid"""
        if pos == wx.DefaultPosition:
            pos = wx.GetMousePosition()
        point = self.ScreenToClient(pos)
        point = self.CalcUnscrolledPosition(point)
        if normalize:
            point.x -= (point.x % (5 * self._object._zoom))
            point.y -= (point.y % (5 * self._object._zoom))
        return point

    @tran.TransactionalMethod('add class')
    def OnAddClass(self, event):
        """addClass handler"""
        parent = self._object.parent
        topClass = parent.outer_class
        #for the moment we dont allow nested classes
        if topClass is not None:
            parent = topClass.parent
        position = self.GetLocalPosition()
        dialog = dlg.ClassDialog(self, parent)
        if dialog.ShowModal() != wx.ID_OK:
            return False
        cls = model.cc.Class(parent=parent, name=dialog._name,
            template=dialog._template, template_types=dialog._template_types,
            prefix=dialog._memberPrefix, notes=dialog._note)
        # draw in diagram
        self._object.SaveState()
        self._element = self._object.AddClass(cls, position)
        #self._point = self.GetLocalPosition()
        #self._offset = wx.Point(0, 0)
        #self._element = self._object.AddClass(cls, self._point)
        self.SetFocus()
        self._element.Draw(self.AdjustedDC())
        return True

    def OnMouseMove(self, event):
        """Handles mouse move event"""
        #if not self.m_bitmap1.HasCapture():
        #    return
        if self._operation == OP_CLICK_ELEMENT:
            # TransactionStack.DoBeginTransaction("move element")
            # TransactionStack.DoSaveState(self._object)
            self._operation = OP_MOVE_ELEMENT
        if self._operation == OP_CLICK_SEGMENT:
            # TransactionStack.DoBeginTransaction("move segment")
            # TransactionStack.DoSaveState(self._object)
            self._operation = OP_MOVE_SEGMENT
        if self._operation == OP_MOVE_ELEMENT:
            dc = self.AdjustedDC()
            self._point = self.GetLocalPosition()
            self.Freeze()
            shift = self._point + self._offset - self._element._pos
            self._object.EraseSelected(dc)
            self._object.ShiftSelected(shift)
            self._object.DrawClassDiagram(dc)
            self.Thaw()
        if self._operation == OP_MOVE_SEGMENT:
            dc = self.AdjustedDC()
            self._point = self.GetLocalPosition()
            self.Freeze()
            pos = self._point + self._offset
            self._element.Erase(dc)
            self._index = self._element.ShiftSegment(self._index, pos)
            self._object.DrawClassDiagram(dc)
            self.Thaw()
        elif self._operation in (OP_INSERT_CLASS, OP_MOVE_CLASS):
            dc = self.AdjustedDC()
            self._point = self.GetLocalPosition()
            self.Freeze()
            self._element.Erase(dc)
            self._object.MoveElement(self._element, self._point + self._offset)
            self._object.DrawClassDiagram(dc)
            self.Thaw()
        else:
            (obj, where) = self._object.HitTest(self.GetLocalPosition())
            if where is model.IN_SHAPE:
                wx.SetCursor(wx.StockCursor(wx.CURSOR_HAND))
                return
            elif where is model.IN_LINE_MARK:
                segment = obj[1]
                if segment._orientation is wx.HORIZONTAL:
                    wx.SetCursor(wx.StockCursor(wx.CURSOR_SIZENS))
                else:
                    wx.SetCursor(wx.StockCursor(wx.CURSOR_SIZEWE))
            else:
                wx.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))

    def OnMouseDown(self, event):
        "mouse left down"
        self.SetFocus()
        wx.YieldIfNeeded()
        if self._operation is None:
            p = self.GetLocalPosition(False)
            (element, where) = self._object.HitTest(p)
            # some hit-test returns subelements
            if type(element) is tuple:
                subelement = element[1]
                element = element[0]
            else:
                subelement = None
            #new
            dc = self.AdjustedDC()
            if where is not model.NO_WHERE:
                self._object.BringToFront(element)
                if wx.GetKeyState(wx.WXK_CONTROL):
                    element.ToggleSelected()
                    # if there are two classes selected quick show relation dialog
                    if element.Selected():
                        self._element = element
                        self._point = self.GetLocalPosition()
                        if type(element) is model.ClassElement:
                            l = [x for x in self._object.selected_elements if type(x) is model.ClassElement and x != element]
                            if len(l) == 1:
                                other = l[0]
                                # Ok we can popup the relation
                                self.AddRelation(other.object, element.object)
                                self.Freeze()
                                self._object.DrawClassDiagram(dc)
                                self.Thaw()
                                self._operation = None
                                return
                    element.Erase(dc)
                else:
                    self.Freeze()
                    if not wx.GetKeyState(wx.WXK_SHIFT):
                        self._object.EraseSelected(dc)
                        self._object.UnselectAll()
                    element.Select()
                    self._element = element
                    self._point = self.GetLocalPosition()
                if element.Selected():
                    if where is model.IN_LINE_MARK:
                        self._operation = OP_CLICK_SEGMENT
                        self._offset = subelement._mark - self._point
                        self._index = element.segments.index(subelement)
                    else:
                        self._operation = OP_CLICK_ELEMENT
                        self._offset = element._pos - self._point
                else:
                    self._operation = None
            else:
                self.Freeze()
                self._object.EraseSelected(dc)
                self._object.UnselectAll()
            self._object.DrawClassDiagram(dc)
            self.Thaw()

        elif self._operation in (OP_INSERT_CLASS, OP_MOVE_CLASS):
            self._operation = None
            self._object.PositionClass(self._element, self._point)
        event.Skip()

    def OnMouseUp(self, event):
        "mouse left up"
        if self._operation == OP_CLICK_ELEMENT:
            self._operation = None
        elif self._operation in (OP_MOVE_ELEMENT, OP_MOVE_SEGMENT):
            #tran.TransactionStack.DoCommit()
            self._operation = None
        elif self._operation == OP_CLICK_CLASS:
            self._operation = None
            if wx.GetKeyState(wx.WXK_CONTROL):
                self._element.ToggleSelected()
            else:
                self._object.UnselectAll()
                self._element.Select()
            dc = self.AdjustedDC()
            self.Freeze()
            self._object.DrawClassDiagram(dc)
            self.Thaw()
        event.Skip()

    def OnUpdateAddClass(self, event):
        """addClass update"""
        event.Enable(True)

    def OnUpdateSelectClasses(self, event):
        """Update classDiagramPane command"""
        event.Enable(True)

    def OnSelectClasses(self, event):
        """Show dialog for select classes"""
        dlg.SelectClassesDialog(self, self._object).ShowModal()

    @tran.TransactionalMethod('add diagram note')
    def OnDiagramAddNote(self, event):
        """Handles add note to diagram"""
        position = self.GetLocalPosition()
        dialog = dlg.NoteDialog(self)
        parent = self._object.project
        if dialog.ShowModal() != wx.ID_OK:
            return False
        note = model.Note(parent=parent, note=dialog._note)
        # draw in diagram
        self._object.SaveState()
        self._element = self._object.AddNote(note, position)
        self.SetFocus()
        self._element.Draw(self.AdjustedDC())
        return True

        # pending : copy attributes & do rendering
        return False

    def OnUpdateDiagramAddNote(self, event):
        """Handles add note to diagram"""
        return True

    def OnUpdateAddRelation(self, event):
        """Handles add relation to diagram"""
        s = self._element and self._element._obj
        s = s and isinstance(s, model.TComponent
            ) and s.inner_relation_container
        event.Enable(bool(s))

    def OnAddRelation(self, event):
        """Handle add relation method"""
        obj = self._element and self._element._obj
        obj = obj and obj.inner_relation_container
        self.AddRelation(obj)

    @tran.TransactionalMethod('add relation')
    @wxx.CreationDialog(dlg.RelationDialog, model.cc.Relation)
    def AddRelation(self, objFrom, objTo=None):
        """Handle add relation method"""
        return (context.frame, objFrom, objTo)

    def OnUpdateEditProperties(self, event):
        """Handle update edit properties"""
        event.Enable(bool(self.selected and not self.selected._readOnly))

    def OnEditProperties(self, event):
        """Handle edit properties method"""
        app.ui.edit(context.frame, self.selected)



"""Subclass of Threads, which is generated by wxFormBuilder."""

import wx
from activity.files.ui import ui as ui


# Implementing Threads
class Threads(ui.Threads):
    """Pane showing current threads"""

    def __init__(self, parent, handler):
        """Initialize pane"""
        self._handler = handler
        super(Threads, self).__init__(parent)

    # Update data
    def UpdateData(self):
        """Update the threads info"""
        self.Freeze()
        self.m_listThreads.Clear()
        threads = self._handler.threads
        current = None
        # update thread list
        for k in threads:
            thread = threads[k]
            i = self.m_listThreads.Append('{name}:{classname}={id} {status}'.format(
                    name=thread.name, classname=thread.classname, id=thread.id, status=thread.status))
            self.m_listThreads.SetClientData(i, thread)
            if thread.is_current:
                current = thread
                self.m_listThreads.SetItemBackgroundColour(i, wx.Colour(127, 255, 255))
        self.Thaw()
        if current:
            self._handler.m_stack.UpdateData(current)

    # Handlers for Threads events.
    def OnSelectThread(self, event):
        """Select thread"""
        # TODO: Implement OnSelectThread
        pass

    def OnActivateThread(self, event):
        """Activate thread"""
        # TODO: Implement OnActivateThread
        pass

    def OnSelectStackFrame(self, event):
        """Select frame"""
        # TODO: Implement OnSelectStackFrame
        pass

    def OnActivateStackFrame(self, event):
        """Activate stack frame"""
        # TODO: Implement OnActivateStackFrame
        pass




"""Subclass of NewInheritance, which is generated by wxFormBuilder."""

import wx
import wxx
from activity.models.ui import ui as ui
import model
import app.resources as rc


# Implementing NewInheritance
class InheritanceDialog(ui.NewInheritance):
    """
    This dialog allows to setup inheritance from
    another struct or class.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialize dialog"""
        super(InheritanceDialog, self).__init__(parent)
        self._name = ""
        self._original = ""
        self._element = container
        self.project = container.project
        self.__initClassBoxes__()
        self.m_choice11.SetFocus()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("inheritance"))
        self.SetIcon(icon)

    def __initClassBoxes__(self):
        """Initialize class boxes"""
        classes = self.project.level_classes
        scoped = lambda x: (hasattr(x, 'scoped') and x.scoped) or x.name
        for clase in classes:
            if self._element == clase:
                continue
            if self._element.IsAncestor(clase):
                continue
            self.m_choice11.Append(scoped(clase), clase)
        if len(classes) > 0:
            self.m_choice11.SetSelection(0)

    def Validate(self):
        """Validation"""
        iSel = self.m_choice11.GetCurrentSelection()
        if iSel == wx.NOT_FOUND:
            wx.MessageBox("Inheritance requires selecting base class", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        self._name = self.m_choice11.GetString(iSel)
        self._ancestor = self.m_choice11.GetClientData(iSel)
        iSel = self.m_choice12.GetCurrentSelection()
        if iSel == wx.NOT_FOUND:
            wx.MessageBox("Inheritance requires access", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        self._access = self.m_choice12.GetString(iSel)
        self._virtual = self.m_checkBox51.IsChecked()
        self._note = self.m_richText3.GetValue()
        if self._name != self._original:
            inheritance = self._element(model.cc.Inheritance,
                filter=lambda x: x.inner_class == self._element, cut=True)
            if self._name in [x._name for x in inheritance]:
                wx.MessageBox("Inheritance from " + self._name +
                " already exists", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        return True

    def CopyAttributes(self, inheritance):
        """Copy attributes to inheritance"""
        inheritance._name = self._name
        inheritance.SetAncestor(self._ancestor)
        inheritance._access = self._access
        inheritance._virtual = self._virtual
        inheritance.note = self._note

    def SetAttributes(self, inheritance):
        """Setup attributes for editing already inheritance"""
        scoped = lambda x: (hasattr(x, 'scoped') and x.scoped) or x.name
        self._original = scoped(inheritance._ancestor)
        iSel = self.m_choice11.FindString(self._original)
        self.m_choice11.SetSelection(iSel)
        iSel = self.m_choice12.FindString(inheritance._access)
        self.m_choice12.SetSelection(iSel)
        self.m_checkBox51.SetValue(inheritance._virtual)
        self.m_richText3.SetValue(inheritance.note)
        self.SetTitle("Edit inheritance")

    def get_kwargs(self):
        """returns arguments suitable for object instance"""
        return {'parent': self._element, 'ancestor': self._ancestor,
            'access': self._access, 'virtual': self._virtual,
            'note': self._note}

    # Handlers for NewMethod events.
    def OnCancel(self, event):
        """cancel event handler"""
        self.EndModal(wx.ID_CANCEL)

    # Handlers for NewInheritance events.
    def OnOK(self, event):
        """Process OnOk event"""
        """ok event handler"""
        if self.Validate():
            self.EndModal(wx.ID_OK)





# -*- coding: utf-8 -*-
"""Extensions for supporting local filesystem transactional objects.
Objects may be created/modified/deleted."""

#class recycle_bin
import os
import shutil
from tran import TransactionObject


class TransactionFSObject(TransactionObject):
    """This class constitutes the base for filesystem transaction objects.
    The main problem with this element is that we **must** avoid the use of
    relative paths. For this pourpose, we need a base path provider"""
    def __init__(self, **kwargs):
        """init"""
        abspath = kwargs['file']
        #assert os.access(abspath, os.W_OK)  # test for write access
        #assert os.access(abspath, os.R_OK)  # test for read access
        self._mode_dir = os.path.isdir(abspath)
        self._uri = kwargs['uri']
        self._file = os.path.relpath(abspath, self._uri.dir)
        self._version = 0
        super(TransactionFSObject, self).__init__(**kwargs)

    @property
    def abs_file(self):
        """return the absolute file name"""
        try:
            return os.path.join(self._uri.dir, self._file)
        except:
            return self._file

    @property
    def version_file(self):
        """return the current version file"""
        path, name = os.path.split(self.abs_file)
        return os.path.join(path, '.{name}.{version}'.format(name=name, version=self._version))

    @property
    def tmp_file(self):
        """return temporally fname"""
        path, name = os.path.split(self.abs_file)
        return os.path.join(path, '.{name}.tmp'.format(name=name))

    def swap(self, f1, f2):
        """swap files"""
        print "swap {f1}<->{f2}\n".format(f1=f1, f2=f2)
        shutil.copyfile(f1, self.tmp_file)
        shutil.copyfile(f2, f1)
        shutil.copyfile(self.tmp_file, f2)
        os.remove(self.tmp_file)

    def SaveState(self):
        """Save state"""
        # The version of the file store changes and open a new version
        #print "saving state of version : {self._version}\n".format(self=self)
        try:
            if self._mode_dir:
                super(TransactionFSObject, self).SaveState()
                return
            shutil.copyfile(self.abs_file, self.version_file)
            self._version = self._version + 1
        except:
            pass
        super(TransactionFSObject, self).SaveState()

    def OnUndoRedoChanged(self):
        """Do a transactional action"""
        if not self.InUndoRedo():
            return
        if not self._mode_dir:
            try:
                if self.InUndo():
                    if not os.path.exists(self.abs_file):
                        if os.path.exists(self.version_file):
                            os.remove(self.version_file)
                    else:
                        shutil.copyfile(self.abs_file, self.version_file)
                    self._version = self._version - 1
                    if os.path.exists(self.version_file):
                        shutil.copyfile(self.version_file, self.abs_file)
                    else:
                        os.remove(self.abs_file)
                    self._version = self._version + 1
                elif self.InRedo():
                    if os.path.exists(self.version_file):
                        shutil.copyfile(self.version_file, self.abs_file)
                    else:
                        os.remove(self.abs_file)
            except:
                pass

    def OnUndoRedoLoaded(self):
        """Do initial load"""
        print "OnUndoRedoLoaded called"
        self._version = 0

    def OnUndoRedoRemoving(self):
        """Do a transactional action"""
        # this operation is tentative:
        try:
            if self._mode_dir:
                os.rename(self.abs_file, self.tmp_file)
            else:
                shutil.copyfile(self.abs_file, self.version_file)
                os.remove(self.abs_file)
        except:
            pass

    def OnUndoRedoAdd(self):
        """Do a transactional action"""
        if not self.InUndoRedo():
            return
        try:
            if self._mode_dir:
                os.rename(self.tmp_file, self.abs_file)
            else:
                shutil.copyfile(self.version_file, self.abs_file)
        except:
            pass

    def Dispose(self):
        """When an object is finally removed from the stack,
        the associated external resources may be also released."""
        try:
            if self._mode_dir:
                f = self.tmp_file
                if os.access(f, os.W_OK):
                    shutil.rmtree(f)
            else:
                if self._version:
                    os.remove(self.version_file)
        except:
            pass



# -*- coding: utf-8 -*-

"""
This decorator applies to python operations that successfully transact for export files.
This decorator only applies on views that operate over selected elements
"""


def with_python_export(function):
    """method decorator that provides self name access"""
    def wrap(self, *args, **kwargs):
        """wrapped calls"""
        if function(self, *args, **kwargs):
            try:
                s = self.selected
                r = s.inner_module or s.inner_package
                if r:
                    r.ExportPythonCodeFiles()
            except:
                raise ValueError('with_python_export decorator only works on views with selected item')
    return wrap




# -*- coding: utf-8 -*-
import uuid
import app
from ctx import theContext as context
import model
import tran
#import tasks

"""
Created on Sun Dec 15 23:49:51 2013

@author: mel
"""


class TCommon(object):
    """Clase para componentes con soporte transamodelional"""
    task_container = False
    project_container = False
    class_container = False
    context_container = False
    folder_container = False
    diagram_container = False
    module_container = False
    namespace_container = False
    type_container = False
    inheritance_container = False
    member_container = False
    argument_container = False
    relation_container = False
    function_container = False
    variable_container = False
    enum_container = False
    import_container = False
    package_container = False
    library_container = False
    repository_container = False
    dir_container = False
    file_container = False

    #visual methods
    def draggable(self):
        """returns info about if the object can be moved"""
        return not self._readOnly

    def drop(self, to):
        """drop this elemento to another place"""
        return False

    def __init__(self, **kwargs):
        """Initialize a transactional component"""
        self._parent = kwargs.get('parent', None)
        self._name = kwargs.get('name', '')
        self._serial = kwargs.get('serial', False)
        self._note = kwargs.get('note', '')
        self._readOnly = kwargs.get('readonly', False)
        self._visibleInTree = kwargs.get('visibleInTree', True)
        self._declare = kwargs.get('declare', True)
        self._implement = kwargs.get('implement', True)
        self._child = {}
        if self._parent is not None:
            self._parent.addChild(self, kwargs.get('prev', None))
        self._uid = uuid.uuid4()
        super(TCommon, self).__init__(**kwargs)

        # allow to dynamic children creation through kwargs
        child_kwargs = kwargs.get('child_kwargs', None)
        if child_kwargs is not None:
            for t in child_kwargs:
                for chkwargs in child_kwargs[t]:
                    chkwargs['parent'] = self
                    t(**chkwargs)

    @property
    def uid(self):
        """Accessor to the uid of the object"""
        return self._uid

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['parent'] = self._parent
        kwargs['name'] = self._name
        kwargs['serial'] = self._serial
        kwargs['note'] = self._note
        kwargs['readonly'] = self._readOnly
        kwargs['visibleInTree'] = self._visibleInTree
        kwargs['declare'] = self._declare
        kwargs['implement'] = self._implement
        # recursively add child kwargs
        kwargs['child_kwargs'] = dict([(t,
            [x.get_kwargs() for x in self._child[t]]) for t in self._child.keys()])
        return kwargs

    def update_container(self):
        """Update the container info"""
        pass

    def WriteComment(self, pf):
        """Write the note using the writter"""
        #ok, now we place comments, if any
        if len(self._note) > 0:
            pf.writeln("/**")
            txt = self._note
            txt.replace('\r', '')
            lines = txt.split("\n")
            for line in lines:
                line.replace('*/', '* /')
                pf.writeln("* {}".format(line))
            pf.writeln("**/")

    def WriteContextsPrefixDeclaration(self, pf):
        """Write the contexts prefixes for declaration"""
        if hasattr(self, '_contexts'):
            for i in range(0, len(self._contexts)):
                item = self._contexts[i]
                if len(item._prefix_declaration):
                    pf.writeln('{}'.format(item._prefix_declaration))

    def WriteContextsSufixDeclaration(self, pf):
        """Write the contexts sufixes for declaration"""
        if hasattr(self, '_contexts'):
            for i in range(len(self._contexts) - 1, -1, -1):
                item = self._contexts[i]
                if len(item._sufix_declaration):
                    pf.writeln('{}'.format(item._sufix_declaration))

    def WriteContextsPrefixImplementation(self, pf):
        """Write the contexts prefixes for implementation"""
        if hasattr(self, '_contexts'):
            for i in range(0, len(self._contexts)):
                item = self._contexts[i]
                if len(item._prefix_implementation):
                    pf.writeln('{}'.format(item._prefix_implementation))

    def WriteContextsSufixImplementation(self, pf):
        """Write the contexts sufixes for implementation"""
        if hasattr(self, '_contexts'):
            for i in range(len(self._contexts) - 1, -1, -1):
                item = self._contexts[i]
                if len(item._sufix_implementation):
                    pf.writeln('{}'.format(item._sufix_implementation))

    def __call__(self, *args, **kwargs):
        """___call__(type1,...,typeN, filter=lambda x: True, cut=False):
        Returns the list of nested childs of one of the types.

        filter: indicates a boolean filter method that must
                return True for valid elements.

        cut: stops traveling if filter is unsatisfied?
        """
        r = []
        fn = kwargs.get('filter', lambda x: True)
        cut = kwargs.get('cut', False)
        for k in self._child:
            if app.isclass(k, *args):
                r.extend(filter(fn, self[k]))
            if cut:
                for o in filter(fn, self[k]):
                    r.extend(o(*args, **kwargs))
            else:
                for o in self[k]:
                    r.extend(o(*args, **kwargs))
        return r

    @property
    def name(self):
        """Get the name"""
        return self._name

    @property
    def note(self):
        """Get the name"""
        return self._note

    @note.setter
    def note(self, note):
        """Set the note"""
        self._note = note

    @property
    def label(self):
        """Get tree label"""
        return self._name

    @property
    def tree_label(self):
        """Get tree label"""
        return self.label

    @property
    def tab_label(self):
        """Get tree label"""
        return self.label

    #container properties
    @property
    def inner_class_container(self):
        """return the inner class container if any"""
        return (self.class_container and self) or (self.parent and self.parent.inner_class_container)

    @property
    def inner_context_container(self):
        """return the inner context container if any"""
        return (self.context_container and self) or (self.parent and self.parent.inner_context_container)

    @property
    def inner_folder_container(self):
        """return the inner folder container if any"""
        return (self.folder_container and self) or (self.parent and self.parent.inner_folder_container)

    @property
    def inner_repository_container(self):
        """return the inner folder container if any"""
        return (self.repository_container and self) or (self.parent and self.parent.inner_repository_container)

    @property
    def inner_diagram_container(self):
        """return the inner diagram container if any"""
        return (self.diagram_container and self) or (self.parent and self.parent.inner_diagram_container)

    @property
    def inner_module_container(self):
        """return the inner module container if any"""
        return (self.module_container and self) or (self.parent and self.parent.inner_module_container)

    @property
    def inner_task_container(self):
        """return the inner task container if any"""
        return (self.task_container and self) or (self.parent and self.parent.inner_task_container)

    @property
    def inner_type_container(self):
        """return the inner module container if any"""
        return (self.type_container and self) or (self.parent and self.parent.inner_type_container)

    @property
    def inner_enum_container(self):
        """return the inner module container if any"""
        return (self.enum_container and self) or (self.parent and self.parent.inner_enum_container)

    @property
    def inner_namespace_container(self):
        """return the inner namespace container if any"""
        return (self.namespace_container and self) or (self.parent and self.parent.inner_namespace_container)

    @property
    def inner_inheritance_container(self):
        """return the inner ineritance container if any"""
        return (self.inheritance_container and self) or (self.parent and self.parent.inner_inheritance_container)

    @property
    def inner_member_container(self):
        """return the inner member container"""
        return (self.member_container and self) or (self.parent and self.parent.inner_member_container)

    @property
    def inner_argument_container(self):
        """return the inner argument container"""
        return (self.argument_container and self) or (self.parent and self.parent.inner_argument_container)

    @property
    def inner_package_container(self):
        """return the inner package container"""
        return (self.package_container and self) or (self.parent and self.parent.inner_package_container)

    @property
    def inner_project_container(self):
        """return the inner project container"""
        return (self.project_container and self) or (
            self.parent and self.parent.inner_project_container)

    @property
    def inner_relation_container(self):
        """return the inner relation container"""
        return (self.relation_container and self) or (
            self.parent and self.parent.inner_relation_container)

    @property
    def inner_function_container(self):
        """return the inner function container"""
        return (self.function_container and self) or (self.parent and self.parent.inner_function_container)

    @property
    def inner_variable_container(self):
        """return the inner relation container"""
        return (self.variable_container and self) or (self.parent and self.parent.inner_variable_container)

    @property
    def inner_import_container(self):
        """return the inner relation container"""
        return (self.import_container and self) or (self.parent and self.parent.inner_import_container)

    def inner(self, t, self_include=True):
        """Search through ancestors the first ancestor of some type"""
        if self_include:
            if (type(t) is list and app.isclass(type(self), *t)) or app.isclass(type(self), t):
                return self
        return self._parent and self._parent.inner(t)

    @property
    def can_delete(self):
        """Check about if a component can be deleted"""
        return not self._readOnly

    @property
    def read_only(self):
        """get read only status"""
        return self._readOnly

    def __setitem__(self, cls, listobj):
        """sets the list of some class childs"""
        assert type(listobj) is list
        self._child[cls] = listobj

    def __getitem__(self, cls):
        """returns the list of childs with some class"""
        if not cls in self._child:
            self._child[cls] = []
        return self._child[cls]

    @property
    def project(self):
        """return the container project"""
        return self.inner(model.Project)

    @property
    def workspace(self):
        """return the container project"""
        return self.inner(model.Workspace)

    @property
    def inner_types_folder(self):
        """return the container types folder"""
        return self.inner(model.cc.TypesFolder)

    @property
    def classes(self):
        """returns the list of all classes"""
        return self(model.cc.Class)

    @property
    def diagrams(self):
        """returns the list of all diagrams"""
        return self(model.ClassDiagram)

    @property
    def namespaces(self):
        """returns the list of all namespaces"""
        return self(model.cc.Namespace)

    @property
    def folders(self):
        """returns the list of all folder"""
        return self(model.Folder)

    @property
    def modules(self):
        """returns the list of all modules"""
        return self(model.cc.Module, model.py.Module)

    @property
    def packages(self):
        """returns the list of all packages"""
        return self(model.py.Package)

    @property
    def functions(self):
        """returns the list of all functions"""
        return self(model.cc.Function)

    @property
    def constructors(self):
        """returns the list of all functions"""
        return self(model.cc.Constructor)

    @property
    def destructors(self):
        """returns the list of all functions"""
        return self(model.cc.Destructor)

    @property
    def methods(self):
        """returns the list of all functions"""
        return self(model.cc.MemberMethod)

    @property
    def variables(self):
        """returns the list of all variables"""
        return self(model.cc.Data, True)

    @property
    def inner_class(self):
        """returns the innermost class"""
        return self.parent and self.parent.inner_class

    @property
    def inner_namespace(self):
        """returns the innermost class"""
        return self.inner(model.cc.Namespace)

    @property
    def inner_library(self):
        """returns the inner library if any"""
        return self.inner(model.Library)

    @property
    def inner_folder(self):
        """returns the innermost folder"""
        return self.inner(model.Folder)

    @property
    def inner_module(self):
        """returns the innermost class"""
        return self.inner([model.cc.Module, model.py.Module])

    @property
    def inner_package(self):
        """returns the innermost class"""
        return self.inner(model.py.Package)

    @property
    def inner_method(self):
        """returns the innermost folder"""
        return self.inner([model.cc.MemberMethod, model.cc.IsClassMethod])

    @property
    def inner_function(self):
        """returns the innermost folder"""
        return self.inner(model.cc.Function)

    @property
    def inner_constructor(self):
        """returns the innermost folder"""
        return self.inner(model.cc.Constructor)

    @property
    def outer_class(self):
        """returns the outermost class"""
        return self.parent and self.parent.outer_class

    @property
    def outer_module(self):
        """returns the outermost module"""
        return (self.parent and self.parent.outer_module) or ((
            isinstance(self, (model.cc.Module, model.py.Module)) or None) and self)

    @property
    def outer_namespace(self):
        """returns the outermost namespace"""
        return (self.parent and self.parent.outer_namespace) or ((
            isinstance(self, model.cc.Namespace) or None) and self)

    @property
    def outer_folder(self):
        """returns the outermost folder"""
        return (self.parent and self.parent.outer_folder) or ((
            isinstance(self, model.Folder) or None) and self)

    @property
    def types(self):
        """This method gets the list of visible types"""
        return (self.outer_class or self.parent).types

    @property
    def level_classes(self):
        """return the list of classes inside this level"""
        cls = self.outer_class
        return self(model.cc.Class, model.py.Class, filter=lambda x: x.parent.outer_class == cls)

    @property
    def top_namespaces(self):
        """return the list of top namespaces"""
        return [x for x in self.namespaces if x.outer_namespace == x]

    @property
    def top_folders(self):
        """return the list of top folders"""
        return [x for x in self.folders if x.outer_folder == x]

    @property
    def parent(self):
        """return the parent object"""
        return self._parent

    @property
    def path(self):
        """returns the ordered parents list"""
        return ((self.parent and self.parent.path) or []) + [self]

    @property
    def scope(self):
        """Get the scope"""
        return (self._parent and self._parent.scope) or ''

    @property
    def scoped_name(self):
        """returns the absolute name, with scope"""
        return '{self.scope}{self._name}'.format(self=self)

    @property
    def template_types(self):
        """Returns the list of nested template types"""
        return (self.parent and self.parent.template_types) or []

    def addChild(self, obj, prev=None):
        """Adds a child member to dictionary"""
        cls = type(obj)
        if cls not in self._child:
            self._child[cls] = [obj]
        elif obj not in self._child[cls]:
            ls = self._child[cls]
            if prev is not None:
                if prev in ls:
                    ls.insert(ls.index(prev) + 1, obj)
                else:
                    ls.insert(0, obj)
            else:
                ls.append(obj)

    def removeChild(self, obj):
        """Remove a child member from dictionary"""
        cls = type(obj)
        if cls in self._child and obj in self._child[cls]:
            self._child[cls].remove(obj)

    def hasChild(self, **kwargs):
        """Checks the existence of some child"""
        recurse = kwargs.get('recurse', True)
        obj = kwargs.get('obj', None)
        if obj is not None:
            t = type(obj)
            if not recurse:
                return obj in self[t]
        else:
            t = kwargs.get('type', None)
            name = kwargs.get('name', None)
            if not recurse:
                return name in [x._name for x in self[t]]
        for c, l in [(u, v) for u, v in self._child.items()
            if hasattr(u, 'hasChild')]:
            for x in l:
                if x.hasChild(**kwargs):
                    return True
        return False

    def __getstate__(self):
        """Set picke context"""
        state = dict(self.__dict__)
        if '_parentIndex' in state:
            del state['_parentIndex']
        if '_pane' in state:
            del state['_pane']
        if '_paneIndex' in state:
            del state['_paneIndex']
        return state

    def Delete(self):
        """Remove all the elements"""
        self.DeleteChilds()
        super(TCommon, self).Delete()

    def DeleteChilds(self):
        """Remove all the child elements"""
        items = self._child.keys()
        for cl in items:
            rm = [x for x in self._child[cl]]
            for x in rm:
                x.Delete()

    def RemoveRelations(self):
        """Utility for undo/redo"""
        if self._parent is not None:
            self._parent.removeChild(self)

    def RestoreRelations(self):
        """Utility for undo/redo"""
        if self._parent is not None:
            self._parent.addChild(self)

    def OnUndoRedoChanged(self):
        """Update from app"""
        super(TCommon, self).OnUndoRedoChanged()
        self.update_container()
        context.RenderUndoRedoChanged(self)

    def OnUndoRedoRemoving(self, root=True):
        """Prepare for delete"""
        super(TCommon, self).OnUndoRedoRemoving()
        for cl, ls in self._child.items():
            for x in ls:
                x.OnUndoRedoRemoving(False)
        if root and self.parent is not None:
            l = self.parent[type(self)]
            if self in l:
                self._parentIndex = l.index(self)
                l.pop(self._parentIndex)
        context.RenderUndoRedoRemoving(self)

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        # Our first step is check if element was created first and must
        # be repositioned in concrete place of his parent.
        super(TCommon, self).OnUndoRedoAdd()
        if self._parent is not None and hasattr(self, '_parentIndex'):
            l = self._parent[type(self)]
            if self in l:
                l.remove(self)
            l.insert(self._parentIndex, self)
            delattr(self, '_parentIndex')
        # We can have a context rendering
        context.RenderUndoRedoAdd(self)
        rlc = list(reversed(list(self._child.keys())))
        for c in rlc:
            rlo = list(reversed(self._child[c]))
            for o in rlo:
                o.OnUndoRedoAdd()

    def OnUndoRedoLoaded(self):
        """Restore object after loading"""
        context.RenderLoadedAdd(self)
        rlc = list(reversed(list(self._child.keys())))
        for c in rlc:
            rlo = list(reversed(self._child[c]))
            for o in rlo:
                o.OnUndoRedoLoaded()

    def OnUndoRedoUnloaded(self):
        """Restore object after loading"""
        rlc = list(reversed(list(self._child.keys())))
        for c in rlc:
            rlo = list(reversed(self._child[c]))
            for o in rlo:
                o.OnUndoRedoUnloaded()
        context.RenderUndoRedoRemoving(self)




# -*- coding: utf-8 -*-
from Argument import Argument
from Namespace import Namespace
from Class import Class

from Constructor import Constructor
from Destructor import Destructor
from Inheritance import Inheritance
from MemberData import MemberData
from MemberMethod import MemberMethod
from Relation import Relation
from Relation import RelationFrom
from Relation import RelationTo
from Type import Type
from Type import typeinst
#from TypesFolder import name
from TypesFolder import TypesFolder
from Module import Module
from Function import Function
from Data import Data
from Enum import Enum
from IsClassMethod import IsClassMethod
from InitMethod import InitMethod
from ExitMethod import ExitMethod
from Friendship import Friendship
from pyBoostModule import pyBoostModule



"""Subclass of NewType, which is generated by wxFormBuilder."""

import re
import wx
import wxx
from activity.models.ui import ui as ui
import model
import app.resources as rc


class TypeDialog(ui.NewType):
    """
    This dialog allows to create a base type
    in a very liberal way.
    Be careful!
    This method of declaring a type, is planned
    to remove as soon as the support of the C++
    syntax be complete.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Init dialog"""
        super(TypeDialog, self).__init__(parent)
        self._container = container
        self.original = ""
        self.m_textCtrl2.SetFocus()
        if container.inner_class:
            self.m_access.Enable()
            self.m_accessLb.Enable()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("type"))
        self.SetIcon(icon)

    def Validate(self):
        """Process OnOk event"""
        self._name = self.m_textCtrl2.GetValue()
        if len(self._name) == 0:
            wx.MessageBox("Type name must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        s = self._name.replace('::', '__')
        if re.match("^[A-Za-z_][0-9A-Za-z_]*$", s) is None:
            wx.MessageBox("Type name contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if self.original != self._name:
            types = self._container(model.cc.Type)
            if self._name in [x._name for x in types]:
                wx.MessageBox("Type already exist", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        if  self.m_checkBoxTemplate.GetValue():
            self._template = self.m_template.GetValue()
        else:
            self._template = None
        self._definition = self.m_richText2.GetValue()
        self._note = self.m_richText3.GetValue()
        iSel = self.m_access.GetCurrentSelection()
        if iSel == wx.NOT_FOUND:
            wx.MessageBox("Invalid access", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        self._access = self.m_access.GetString(iSel)
        return True

    def CopyAttributes(self, _type):
        """Copy dialog attributes to object"""
        _type._name = self._name
        _type._definition = self._definition
        _type._note = self._note
        _type._access = self._access
        if  self.m_checkBoxTemplate.GetValue():
            _type._template = self.m_template.GetValue()
        else:
            _type._template = None

    def SetAttributes(self, _type):
        """Transfer attributes from existing type"""
        self._container = _type.parent
        self._name = _type._name
        self.original = _type._name
        self._definition = _type._definition
        self._note = _type._note
        self._access = _type._access
        iSel = self.m_access.FindString(_type._access)
        self.m_access.SetSelection(iSel)
        self.m_textCtrl2.SetValue(self._name)
        self.m_richText2.SetValue(self._definition)
        self.m_richText3.SetValue(self._note)
        if not _type._template is None:
            self.m_checkBoxTemplate.SetValue(True)
            self.m_stStartTemplate.Enable(True)
            self.m_template.Enable(True)
            self.m_stEndTemplate.Enable(True)
            self.m_template.SetValue(_type._template)
        else:
            self.m_checkBoxTemplate.SetValue(False)
            self.m_stStartTemplate.Enable(False)
            self.m_template.Enable(False)
            self.m_stEndTemplate.Enable(False)
            self.m_template.SetValue('')
        self.SetTitle("Edit type")

    def OnToggleTemplate(self, event):
        """Actualiza el dialogo al hacer toggle del checkbox de template"""
        if self.m_checkBoxTemplate.GetValue():
            self.m_stStartTemplate.Enable(True)
            self.m_template.Enable(True)
            self.m_stEndTemplate.Enable(True)
        else:
            self.m_stStartTemplate.Enable(False)
            self.m_template.Enable(False)
            self.m_stEndTemplate.Enable(False)
        event.Skip()

    def get_kwargs(self):
        """Returns arguments suitable for object creation"""
        kwargs = {}
        kwargs['parent'] = self._container
        kwargs['name'] = self._name
        kwargs['definition'] = self._definition
        kwargs['note'] = self._note
        kwargs['template'] = self._template
        kwargs['access'] = self._access
        return kwargs

    def OnOK(self, event):
        """Handle OnOk command"""
        if self.Validate():
            self.EndModal(wx.ID_OK)





# -*- coding: utf-8 -*-
"""
Created on Mon Dec 16 08:28:25 2013

@author: mel
"""
import model.decorator as ctx
import app.resources as rc
import tran
import model
from tran import TransactionStack


class MemberData(model.Member):
    """Implements member data"""
    context_container = True

    # visual methods
    @tran.TransactionalMethod('move member {}')
    def drop(self, to):
        """Drops datamember inside project or another folder """
        target = to.inner_member_container
        if not target or self.inner_class != target.inner_class or self.project != target.project:
            return False  # avoid move classes between projects
        index = 0
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        "Initialization"
        self._typei = kwargs['type']
        self._access = kwargs.get('access', "public")
        self._static = kwargs.get('static', False)
        self._volatile = kwargs.get('volatile', False)
        self._mutable = kwargs.get('mutable', False)
        self._bitField = kwargs.get('bitfield', False)
        self._bitFieldSize = kwargs.get('bitfieldsize', 0)
        self._default = kwargs.get('default', "")
        super(MemberData, self).__init__(**kwargs)
        if kwargs.get('update_ctors', True):
            self.inner_class.AutoArgs()
            self.inner_class.AutoInit()
        k = self.outer_class or self.outer_module
        if k:
            k.ExportCppCodeFiles(force=True)

    def Delete(self):
        """Handle delete"""
        k = self.outer_class or self.outer_module
        super(MemberData, self).Delete()
        if k:
            k.ExportCppCodeFiles(force=True)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['type'] = self._typei
        kwargs['access'] = self._access
        kwargs['static'] = self._static
        kwargs['volatile'] = self._volatile
        kwargs['mutable'] = self._mutable
        kwargs['bitfield'] = self._bitField
        kwargs['bitfieldsize'] = self._bitFieldSize
        kwargs['default'] = self._default
        kwargs.update(super(MemberData, self).get_kwargs())
        return kwargs

    @ctx.ContextDeclaration()
    def WriteDeclaration(self, pf):
        """Write the member declaration"""
        self.inner_class._ensure_access(pf, self._access)
        self.WriteComment(pf)
        pf.writeln("{};".format(self.label))

    @ctx.ContextImplementation()
    def WriteCode(self, pf):
        """Write the member implementation"""
        if self._static:
            pf.writeln("{};".format(self.GetStaticDefinition()))

    def GetInitializer(self):
        """Return the initializer sequence"""
        if len(self._default) > 0:
            return self._default
        return self._name

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("member", self._access)

    def GetStaticDefinition(self):
        """Return global, static definition"""
        if not self._static:
            return ""
        stype = self._typei.scoped_str([self.inner_class.scope])
        if self._volatile:
            stype = "volatile " + stype
        if self._mutable:
            stype = "mutable " + stype
        #if self._static:
        #    stype = "static " + stype
        epi = ""
        if self._bitField:
            epi = ":" + str(self._bitFieldSize)
        mname = self.inner_class._memberPrefix + self._name
        return stype.format('{self.scope}::{mname}{epi}'.format(
            self=self, mname=mname, epi=epi)) + ' = ' + self._default

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(MemberData, self).OnUndoRedoAdd()

    def OnUndoRedoChanged(self):
        """Make changes in the model as result of change"""
        super(MemberData, self).OnUndoRedoChanged()
        if not TransactionStack.InUndoRedo():
            k = self.outer_class or self.outer_module
            if k:
                k.ExportCppCodeFiles(force=True)

    @property
    def label(self):
        """Get tree label"""
        s = ''
        s = self._typei.scoped_str([self.inner_class.scope])
        if self._volatile:
            s = "volatile " + s
        if self._mutable:
            s = "mutable " + s
        if self._static:
            s = "static " + s
        epi = ""
        if self._bitField:
            epi = ":" + str(self._bitFieldSize)
        return s.format(self.prefixed_name) + " " + epi

    @property
    def prefixed_name(self):
        """local var name, with class prefix"""
        return self.inner_class._memberPrefix + self._name

    @property
    def type_instance(self):
        """return the type instance"""
        return self._typei



# -*- coding: utf-8 -*-
"""
Created on Sun Dec 15 23:24:30 2013

@author: mel
"""

import os
import app
import model
import tran
from model.writer import writer
import app.resources as rc
from tran import TransactionStack


class Module(model.TComponent):
    """Implements a Module representation"""
    class_container = False
    folder_container = True
    diagram_container = True
    namespace_container = True
    function_container = True
    variable_container = True
    enum_container = True

    def __init__(self, **kwargs):
        """Initialization"""
        self._header = kwargs.get('header', None)
        self._source = kwargs.get('source', None)
        self._lastSrcTime = None
        self._lastHdrTime = None
        self._header_obj = None  # vincle with file
        self._source_obj = None  # vincle with file
        super(Module, self).__init__(**kwargs)
        self.ExportCppCodeFiles(force=True)

    def Delete(self):
        """Handle delete"""
        project = self.project
        super(Module, self).Delete()
        project.ExportCppCodeFiles(force=True)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        return super(Module, self).get_kwargs()

    def OnUndoRedoRemoving(self):
        """Handle OnUndoRedoRemoving, prevent generating files"""
        stack = tran.TransactionStack
        method = self.ExportCppCodeFiles.inner
        if method not in stack.delayedCallsFiltered:
            stack.delayedCallsFiltered.append(method)
        super(Module, self).OnUndoRedoRemoving()

    @tran.DelayedMethod
    def ExportCppCodeFiles(self, force=True):
        """Do code generation"""
        self.updateSources(force)
        self.updateHeaders(force)

    def WriteCode(self, pf):
        """Write code for module"""
        #first the variables
        for variable in self.variables:
            variable.WriteCode(pf)
        #next functions
        for function in self.functions:
            function.WriteCode(pf)

    def updateSources(self, force=False):
        """does source generation"""
        sources_dir = self.project.sources_dir
        pass
        # we need to do a lot of work here. The name may be from existent source
        # and then, the class is resposible for holding the generation
        fname = os.path.realpath(os.path.join(sources_dir, self._source))
        regenerate = False
        if force or not os.path.exists(fname) or self._lastSrcTime is None:
            regenerate = True
        elif os.path.getmtime(fname) < self._lastSrcTime:
            regenerate = True
        if not regenerate:
            return
        f = open(fname, 'w')
        pf = writer.for_file(f)
        #master include file
        prj = self.project

        pf.writeln("// {user.before.include.begin}")
        pf.writenl()
        pf.writeln("// {user.before.include.end}")
        pf.writenl()
        if prj._useMaster:
            pf.writeln('#include "{}"'.format(prj._masterInclude))
        else:
            pf.writeln('#include "{}.h"'.format(self._header))
        pf.writenl()
        pf.writeln("// {user.before.code.begin}")
        pf.writenl()
        pf.writeln("// {user.before.code.end}")
        pf.writenl()
        #self.WriteCode(pf)
        pf.writenl()
        pf.writeln("// {user.after.code.begin}")
        pf.writenl()
        pf.writeln("// {user.after.code.end}")
        pf.writenl()
        f.close()

    def updateHeaders(self, force=False):
        """Realiza la generacion de fuentes"""
        headers_dir = self.project.headers_dir
        pass
        # we need to do a lot of work here. The name may be from existent header
        # and then, the class is resposible for holding the generation

        fname = os.path.realpath(os.path.join(headers_dir, self._header))
        regenerate = False
        if force or not os.path.exists(fname) or self._lastHdrTime is None:
            regenerate = True
        elif os.path.getmtime(fname) < self._lastHdrTime:
            regenerate = True
        if not regenerate:
            return
        f = open(fname, 'w')
        pf = writer.for_file(f)
        inlines = []
        #create safeguard
        safeguard = self._header.upper() + "_H_INCLUDED"
        pf.writeln("#if !defined({})".format(safeguard))
        pf.writeln("#define {}".format(safeguard))

        #we are using master include? if not, include inheritances
        pf.writenl()
        #ok, now we place comments, if any
        if len(self._note) > 0:
            pf.writeln("/**")
            txt = self._note
            txt.replace('\r', '')
            lines = txt.split("\n")
            for line in lines:
                line.replace('*/', '* /')
                pf.writeln("* {}".format(line))
            pf.writeln("**/")
        #write variables (not static)

        #write methods

        pf.writenl()
        #end safeguard
        pf.writeln("#endif //{0}".format(safeguard))
        #write inlines
        pf.writenl()
        if len(inlines) > 0:
            pf.writeln("#if defined(INCLUDE_INLINES)")
            pf.writeln("#if !defined(INCLUDE_INLINES_{name})".format(name=self._header.upper()))
            pf.writeln("#define INCLUDE_INLINES_{name}".format(name=self._header.upper()))
            pf.writenl()
            #write inlines here

            pf.writeln("#endif //(INCLUDE_INLINES_{name})".format(name=self._header.upper()))
            pf.writeln("#endif //INCLUDE_INLINES")
            pf.writenl()
        self._lastHdrTime = os.path.getmtime(fname)

        pf.writenl()
        f.close()

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("module")

    @property
    def inner_class(self):
        """Get the innermost class container"""
        return None

    def OnUndoRedoChanged(self):
        """Update from app"""
        super(Module, self).OnUndoRedoChanged()
        if not TransactionStack.InUndoRedo():
            self.project.ExportCppCodeFiles(force=True)



"""Subclass of MainWindow, which is generated by wxFormBuilder."""

import os
import signal
import re
import wx
import wx.html2
import wxx
import model
import app
import tran

from ctx import localcontext as context
import app.proCxxGUI as base
import app.resources as rc
import app
import activity
import ostools
import plugin.tools

#LOG_FatalError
#LOG_Error
#LOG_Warning
#LOG_Message
#LOG_Status
#LOG_Info
#LOG_Debug
#LOG_Trace
#LOG_Progress
#LOG_User
#LOG_Max


BASE_VIEW_COMMAND_ID = wx.ID_HIGHEST + 1000
TOOL_VIEW_COMMAND_ID = wx.ID_HIGHEST + 2000

#def redirectFocusWindow(method):
#    """decorador for redirecting event to focus window"""
#    def wrapped(self, event):
#        """decorator method"""
#        wnd = self.FindFocus()
#        if wnd != self:
#            wnd.ProcessEvent(event)
#        else:
#            self.method(event)
#    return wrapped

SKIP_EVENT_RECURSE = True


class mainWindow(base.MainWindow):
    """The main application window"""

    tool_id = TOOL_VIEW_COMMAND_ID

    def __init__(self, app):
        """Initialization of main window"""
        super(mainWindow, self).__init__(None)

        self.imglist = rc.GetBitmapImageList()
        self.viewBook.SetImageList(self.imglist)
        self.docBook.SetImageList(ostools.MimeHandler.imageList)
        wx.Log.SetActiveTarget(wx.LogTextCtrl(self.m_toolLog))
        self._output = wx.LogTextCtrl(self.auxOutput)
        self._output.SetLogLevel(wx.LOG_Progress)

        self._menus = {}
        self._toolsMenus = {}

        context.SetFrame(self)

        self._import_views()
        self._import_tools()


        # need for acceletator message dispatch

        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("project"))
        self.SetIcon(icon)

        #self.Freeze()
        #self.Maximize()

        self._bind_events()

        self.addWelcome()

        #do self update check
        self._tools[plugin.tools.check_updates].doUpdate(True)

    def _bind_events(self):
        """Bind custom events"""
        self.Bind(wxx.EVT_LOGGER, self.OnOutputMessage)
        self.Bind(wxx.EVT_SEARCH, self.OnSearchResults)

    def new_tool_id(self):
        """returns new tool id"""
        value = self.tool_id
        self.tool_id = self.tool_id + 1
        return value

    def _import_views(self):
        """Reads the views definitions and prepare to handle them"""
        global BASE_VIEW_COMMAND_ID
        # initialize dict show
        self._views = dict([(cls, None) for cls in activity.ALL])
        # initialize dict hidden
        self._hviews = {}
        # initialize methods for each view
        import new
        i = BASE_VIEW_COMMAND_ID
        for cls in self._views:
            name = cls.name
            #prepare update event handler
            code = eval('lambda x, event: x.OnUpdateToggleView(activity.INDEX["{name}"], event)'.format(
                name=name))
            u_method = new.instancemethod(code, self, mainWindow)
            fname = 'OnUpdateToggle{name}'.format(name=cls.name)
            setattr(self, fname, u_method)
            u_method = eval('self.{fname}'.format(fname=fname))

            #prepare command event handler
            code = eval('lambda x, event: x.OnToggleView(activity.INDEX["{name}"], event)'.format(
                name=name))
            c_method = new.instancemethod(code, self, mainWindow)
            fname = 'OnToggle{name}'.format(name=cls.name)
            setattr(self, fname, c_method)
            c_method = eval('self.{fname}'.format(fname=fname))

            #add menu entry
            mni = wx.MenuItem(self.menuView, wx.ID_ANY,
                u"Show {name}".format(name=cls.name),
                u"show the view", wx.ITEM_NORMAL)
            mni.SetBitmap(cls.bitmap())
            self.menuView.InsertItem(0, mni)

            #add the menu handlers
            self.Bind(wx.EVT_UPDATE_UI, u_method, id=mni.GetId())
            self.Bind(wx.EVT_MENU, c_method, id=mni.GetId())
            i = i + 1

    def _import_tools(self):
        """Reads the tools definitions and import them"""
        self._tools = dict([(cls, None) for cls in plugin.tools.ALL])
        for tool in self._tools:
            tool_instance = tool.load()
            self.PushEventHandler(tool_instance)
            self._tools[tool] = tool_instance

    def SaveViews(self):
        """Store open views in config"""
        config = context.config
        # store selected view
        i = self.viewBook.GetSelection()
        activeView = (i != wx.NOT_FOUND and self.viewBook.GetPage(i).name) or ''
        config.Write('/view/active', activeView)
        for v in activity.ALL:
            config.Write('/views/perspectives/{}'.format(v.name), v.perspective)
            if v in self._views and self._views[v]:
                config.WriteInt('/views/states/{}'.format(v.name), 1)
            else:
                config.WriteInt('/views/states/{}'.format(v.name), 0)
        last = self.m_mgr.SavePerspective()
        config.Write('/frame/perspectives/current', last)

    def LoadViews(self):
        """Load open views in config"""
        self._mainPerspective = self.m_mgr.SavePerspective()
        config = context.config
        activeView = config.Read('/view/active', '')
        for v in activity.ALL:
            v.perspective = config.Read('/views/perspectives/{}'.format(v.name), '')
            if config.ReadInt('/views/states/{}'.format(v.name), 1):
                self.OpenView(v, bool(activeView == v.name))
        last = config.Read('/frame/perspectives/current', '')
        if last:
            self.m_mgr.LoadPerspective(last)

    def GetViewsStatus(self):
        """Get extra info about view status to be stored as session"""
        status = {}
        for v in activity.ALL:
            if v not in self._views:
                continue
            obj = self._views[v]
            if not obj or not hasattr(obj, 'GetViewStatus'):
                continue
            status[v.name] = obj.GetViewStatus()
        return status

    def SetViewsStatus(self, info):
        """Set extra info about view status to be stored as session"""
        for v in activity.ALL:
            if v not in self._views or v.name not in info:
                continue
            obj = self._views[v]
            if not obj or not hasattr(obj, 'SetViewStatus'):
                continue
            obj.SetViewStatus(info[v.name])
        return True

    def LoadMRU(self):
        """Load MRU"""
        mru = context.mru
        mru.UseMenu(self.menuMRU)
        mru.AddFilesToMenu()

    def addWelcome(self):
        """This adds the welcome page"""
        self.m_startPanel = wx.Panel(self.docBook, wx.ID_ANY,
            wx.DefaultPosition, wx.Size(-1, -1), wx.TAB_TRAVERSAL)
        self.m_startPanel.SetExtraStyle(wx.WS_EX_PROCESS_UI_UPDATES)
        self.m_startPanel.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseStartPage)
        self.m_startPanel.Bind(wx.EVT_PAINT, self.OnPaintStartPage)
        self.docBook.AddPage(self.m_startPanel, u"Welcome!", False,
            rc.GetBitmapIndex('mini_logo'))

    def redirectedToFocus4Enabled(self, event):
        """Update new project"""
        global SKIP_EVENT_RECURSE
        if not SKIP_EVENT_RECURSE:
            event.Enable(False)
            return
        SKIP_EVENT_RECURSE = False
        wnd = self.FindFocus()
        if wnd and wnd != self:
            if wnd.ProcessEvent(event):
                SKIP_EVENT_RECURSE = True
                return
        event.Enable(False)
        SKIP_EVENT_RECURSE = True

    def redirectedToFocus(self, event):
        """Redirect command to parent"""
        wnd = self.FindFocus()
        if wnd and wnd != self:
            wnd.ProcessEvent(event)

    def OnEditUndo(self, event):
        """Implements undo command"""
        wx.LogMessage('undo {}'.format(tran.TransactionStack.instance.UndoName()))
        tran.TransactionStack.DoUndo()

    def OnUpdateEditUndo(self, event):
        """Handle update edit undo"""
        if tran.TransactionStack.instance.CanUndo():
            event.Enable(True)
            name = tran.TransactionStack.instance.UndoName()
            if not name is None:
                event.SetText("Undo " + name + "\tCtrl+Z")
            else:
                event.SetText("Undo" + "\tCtrl+Z")
        else:
            event.Enable(False)
            event.SetText("Undo" + "\tCtrl+Z")

    def OnEditRedo(self, event):
        """Implements redo command"""
        wx.LogMessage('redo {}'.format(tran.TransactionStack.instance.RedoName()))
        tran.TransactionStack.DoRedo()

    def OnUpdateEditRedo(self, event):
        """Handle update edit redo"""
        if tran.TransactionStack.instance.CanRedo():
            event.Enable(True)
            name = tran.TransactionStack.instance.RedoName()
            if not name is None:
                event.SetText("Redo " + name + "\tCtrl+Y")
            else:
                event.SetText("Redo" + "\tCtrl+Y")
        else:
            event.Enable(False)
            event.SetText("Redo" + "\tCtrl+Y")

    def OpenView(self, cls, select=True):
        """Open generic class"""
        if self._views.get(cls, None):
            return
        view = self._hviews.get(cls, None)
        if view is None:
            view = cls(self)
        else:
            del self._hviews[cls]
        self._views[cls] = view
        if wx.__version__ >= '3.0.0.0':
            self.viewBook.AddPage(view, cls.name,
                False, cls.bitmap_index())
        else:
            self.viewBook.AddPage(view, cls.name,
                False, cls.bitmap())
        if select:
            self.viewBook.SetSelection(
                self.viewBook.GetPageIndex(view))
        if self.viewBook.GetSelection() == self.viewBook.GetPageIndex(view):
            if hasattr(view, 'NotifyShow'):
                view.NotifyShow()

    def CloseView(self, cls):
        """close some generic view"""
        view = self._views.get(cls, None)
        if view:
            i = self.viewBook.GetPageIndex(view)
            if i is not wx.NOT_FOUND:
                if i == self.viewBook.GetSelection():
                    if hasattr(view, 'NotifyHide'):
                        view.NotifyHide()
                self.viewBook.RemovePage(i)
            self._hviews[cls] = view
            del self._views[cls]

    def OnToggleView(self, cls, event):
        """Toggle some generic view"""
        if self._views.get(cls, None):
            self.CloseView(cls)
        else:
            self.OpenView(cls)

    def OnUpdateToggleView(self, cls, event):
        """Update toggle tasks view"""
        try:
            if self._views.get(cls, None):
                event.SetText('Hide {name}'.format(name=cls.name))
            else:
                event.SetText('Show {name}'.format(name=cls.name))
        except:
            print 'error in {}'.format(cls)

    def AddMainMenu(self, name, menu):
        """Add on-the-fly main menu"""
        if name in self._menus:
            return False
        nmnu = self.m_menubar1.GetMenuCount()
        #skip help
        if self.m_menubar1.Insert(nmnu - 1, menu, name):
            #app.set_menu_handlers(self, menu,
            #    self.redirectedToFocus, self.redirectedToFocus4Enabled)
            self._menus[name] = menu
            return True
        return False

    def AddToolsMenu(self, name, menu):
        """Append a menu entry to tools"""
        if name in self._toolsMenus:
            return False
        nmnu = self.menuTools.GetMenuItemCount()
        if self.menuTools.InsertItem(nmnu, menu):
            self._toolsMenus[name] = menu
            return True
        return False

    def RemoveMainMenu(self, name, menu):
        """Remove existing main menu"""
        if name not in self._menus:
            return False
        mnu = self._menus[name]  # the menu object to be removed
        if mnu != menu:
            return False
        # we seach the menu, not the name
        pairs = self.m_menubar1.GetMenus()
        for i in range(0, len(pairs)):
            if pairs[i][0] == mnu:
                #ok, we find it
                #app.unset_menu_handlers(self, mnu,
                #    self.redirectedToFocus, self.redirectedToFocus4Enabled)
                self.m_menubar1.Remove(i)
                del self._menus[name]
                return True
        return False

    def RemoveToolsMenu(self, name, menu):
        """Remove existing tool menu"""
        if name not in self._toolsMenus:
            return False
        mnu = self._toolsMenus[name]  # the menu object to be removed
        if mnu != menu:
            return False
        # we search the menu, not the name
        index = self.menuTools.FindItem(mnu)
        if index != wx.NOT_FOUND:
            app.unset_menu_handlers(self, menu,
                self.redirectedToFocus, self.redirectedToFocus4Enabled)
            self.menuTools.RemoveItem(mnu)
            del self._toolsMenus[name]
            return True
        return False

    def DoRenderRemoveElement(self, obj):
        """Do remove element in tree"""
        views = self._views
        views.update(self._hviews)
        for cls in views:
            v = views[cls]
            if v and hasattr(v, 'DoRenderRemoveElement'):
                v.DoRenderRemoveElement(obj)

    def DoRenderAddElement(self, obj):
        """Add element to visual"""
        views = self._views
        views.update(self._hviews)
        for cls in views:
            v = views[cls]
            if v and hasattr(v, 'DoRenderAddElement'):
                v.DoRenderAddElement(obj)

    def UpdateElement(self, obj):
        """Update the tree label for a object"""
        if hasattr(obj, '_pane') and obj._pane:
            obj._pane.Refresh()
        views = self._views
        views.update(self._hviews)
        for cls in views:
            v = views[cls]
            if v and hasattr(v, 'UpdateElement'):
                v.UpdateElement(obj)

    def OnSalir(self, event):
        """Ends application"""
        if context.app.ExitEvenModified():
            context.Destroy()
            #sys.exit()

    def Destroy(self):
        """Destroy frame"""
        del self.m_auiToolBarFile
        del self.m_auiToolBarEdit
        views = self._views
        views.update(self._hviews)
        for cls in views:
            v = views[cls]
            if v and hasattr(v, '_delete_toolbars'):
                v._delete_toolbars()
        super(mainWindow, self).Destroy()

    @tran.TransactionalMethod('create workspace {}')
    @wxx.CreationDialog(app.ui.dlg.WorkspaceDialog, model.Workspace)
    def OnNewWorkspace(self, event):
        """Handle new workspace command"""
        return (self, None)

    def OnSettingsBuildTools(self, event):
        """Show build tools dialog"""
        app.ui.dlg.BuildTools(self).ShowModal()

    def OnEraseStartPage(self, event):
        """Erase start page event"""
        sz = self.m_startPanel.GetSize()
        dc = event.GetDC()
        bmp = wx.EmptyBitmap(729, 119)
        bmp.LoadFile('app/res/beatle.jpg', wx.BITMAP_TYPE_JPEG)
        dc.DrawBitmap(bmp, sz.x // 2 - 365, sz.y // 2 - 60)
        del bmp

    def OnPaintStartPage(self, event):
        """Start page paint event"""
        dc = wx.PaintDC(self.m_startPanel)
        #dc.SetBrush(wx.Brush(wx.RED))
        #dc.Clear()
        return

    def OnEditPreferences(self, event):
        """Handle edit preferences"""
        app.ui.dlg.PreferencesDialog(self).ShowModal()

    def OnHelpPython2Reference(self, event):
        """Handle python 2 reference"""
        self.ShowReference('https://docs.python.org/2/', 'python 2 reference')

    def OnHelpCppReference(self, event):
        """Handle python 2 reference"""
        self.ShowReference('http://en.cppreference.com', 'c++ reference')

    def OnHelpBeatsOfCode(self, event):
        """Handle beats of code reference"""
        self.ShowReference('http://beatsofcode.com', 'online tutorials')

    def ShowReference(self, url, title='Help'):
        """Handle help cpp reference"""
        import app.ui.pane as pane
        for index in range(0, self.docBook.GetPageCount()):
            p = self.docBook.GetPage(index)
            if type(p) is not pane.NavigatorPane:
                continue
            # ok, check url
            if p._url != url:
                continue
            self.docBook.SetSelection(index)
            return
        # ok,there are not a navigation page
        p = pane.NavigatorPane(self.docBook, self)
        if wx.__version__ >= '3.0.0.0':
            #self.docBook.Freeze()
            self.docBook.AddPage(p, title, True, rc.GetBitmapIndex("info"))
            #self.docBook.Thaw()
        else:
            self.docBook.AddPage(p, title, True,
                rc.GetBitmap("info"))
        p.NavigateTo(url)

    @tran.TransactionalMethod('open workspace {}')
    def OnOpenWorkspace(self, event):
        """Handle open workspace method"""
        dialog = wx.FileDialog(self, "Select workspace file", wx.EmptyString,
            wx.EmptyString, "Workspace files (*.wrk)|*.wrk", wx.FD_OPEN)
        if dialog.ShowModal() != wx.ID_OK:
            return False
        workspace = context.app.LoadWorkspace(dialog.GetPath())
        if workspace:
            tran.TransactionStack.DoLoad(workspace)
        return True

    @tran.TransactionalMethod('reload last workspace')
    def ReloadLastFile(self):
        """Opens a project file from MRU"""
        mru = context.mru
        if mru.GetCount() == 0:
            return False
        pf = mru.GetHistoryFile(0)
        if len(pf) == 0:
            return False
        import traceback
        import sys
        try:
            if re.match(r"(.)*\.pcc", pf):
                obj = context.app.LoadProject(pf)
                if not isinstance(obj, model.Project):
                    return False
            elif re.match(r"(.)*\.wrk", pf):
                obj = context.app.LoadWorkspace(pf)
                if not isinstance(obj, model.Workspace):
                    return False
            tran.TransactionStack.DoLoad(obj)
            # as special thing, expanding is good...
            #self.m_tree.Expand(obj=obj)
        except Exception as inst:
            traceback.print_exc(file=sys.stdout)
            print type(inst)     # the exception instance
            print inst.args      # arguments stored in .args
            print inst
            return False
        return True

    @tran.TransactionalMethod('open recent')
    def OnMRUFile(self, event):
        """Opens a project file from MRU"""
        mru = context.mru
        pf = mru.GetHistoryFile(event.GetId() - wx.ID_FILE1)
        if len(pf) == 0:
            return False
        import traceback
        import sys
        try:
            if re.match(r"(.)*\.pcc", pf):
                obj = context.app.LoadProject(pf)
                if not isinstance(obj, model.Project):
                    return False
            elif re.match(r"(.)*\.wrk", pf):
                obj = context.app.LoadWorkspace(pf)
                if not isinstance(obj, model.Workspace):
                    return False
            tran.TransactionStack.DoLoad(obj)
        except Exception as inst:
            traceback.print_exc(file=sys.stdout)
            print type(inst)     # the exception instance
            print inst.args      # arguments stored in .args
            print inst
            return False
        return True

    #def ClosePane(self, index):
    #    """This is a programatic page close"""
    #    if index == wx.NOT_FOUND:
    #        return

    def CloseCurrentDocPane(self, event):
        """This is a programatic page close"""
        index = self.docBook.GetSelection()
        page = self.docBook.GetPage(index)
        if hasattr(page, '_object'):
            obj = page._object
            obj._pane = None
            page.Commit()

        # We have a reported issue in ubuntu, with touch solution:
        # (from Manas Pathak on
        # http://thehacklist.blogspot.com.es/
        # 2010/06/resolving-murrinestyledrawbox-assertion.html)
        """When using Emacs 23 and a few other applications
        on Ubuntu running GNOME, I often get a irritatingly cryptic
        error message ** (emacs:25074): CRITICAL **: murrine_style_draw_box:
        assertion `height >= -1' failed. Turns out it has got
        something to do with the theme I'm running.

        Thanks to Alf, it is very easy to get rid of that by tweaking
        Gtk configuration. Modify the entry
        in /usr/share/themes/Ambiance/gtk-2.0/gtkrc
            from GtkRange::trough-under-steppers = 0 to
        GtkRange::trough-under-steppers = 1."""

        from activity.models.ui import pane
        if type(page) in [pane.MethodPane, pane.PyMethodPane,
            pane.ConstructorPane, pane.PyModulePane]:
            page.PreDelete()
        self.docBook.DeletePage(index)
        # after closing the pane, we must set the focus!
        index = self.docBook.GetSelection()
        if index != wx.NOT_FOUND:
            p = self.docBook.GetPage(index)
            if type(p) in [pane.MethodPane, pane.PyMethodPane,
                pane.ConstructorPane, pane.PyModulePane]:
                    p.m_editor.SetFocus()
                    p.Refresh()
                    p.Update()

    def OnViewPageChanged(self, event):
        """update menus"""
        index = self.viewBook.GetSelection()
        if index != wx.NOT_FOUND:
            page = self.viewBook.GetPage(index)
            if hasattr(page, 'NotifyShow'):
                page.NotifyShow()
        event.Skip()  # fails to skip will block event

    def OnViewPageClose(self, event):
        """update menus"""
        index = self.viewBook.GetSelection()
        if index != wx.NOT_FOUND:
            page = self.viewBook.GetPage(index)
            if hasattr(page, 'NotifyHide'):
                page.NotifyHide()
            if type(page) in self._views:
                del self._views[type(page)]
        event.Skip()  # fails to skip will block event

    def OnPageChanged(self, event):
        """update menus"""
        index = self.docBook.GetSelection()
        if index != wx.NOT_FOUND:
            page = self.docBook.GetPage(index)
            if hasattr(page, 'NotifyShow'):
                page.NotifyShow()
        self.m_mgr.Update()
        event.Skip()  # fails to skip will block event

    def OnViewPageChanging(self, event):
        """add menus"""
        index = self.viewBook.GetSelection()
        if index != wx.NOT_FOUND:
            page = self.viewBook.GetPage(index)
            if hasattr(page, 'NotifyHide'):
                page.NotifyHide()
        event.Skip()  # fails to skip will block event

    def OnPageChanging(self, event):
        """remove menus"""
        index = self.docBook.GetSelection()
        if index != wx.NOT_FOUND:
            page = self.docBook.GetPage(index)
            if hasattr(page, 'NotifyHide'):
                page.NotifyHide()
        event.Skip()  # fails to skip will block event

    def OnClosePane(self, event):
        """close pane"""
        index = event.GetSelection()
        page = self.docBook.GetPage(index)
        if hasattr(page, 'PreDelete'):
            page.PreDelete()
        if hasattr(page, '_object'):
            obj = page._object
            obj._pane = None
            page.Commit()
        if  index != self.docBook.GetSelection():
            return
        p = self.docBook.GetPage(index)
        if hasattr(p, 'm_editor'):
            p.m_editor.SetFocus()
        p.Refresh()
        p.Update()
        event.Skip()

    def OpenDoc(self, _class, obj):
        """Opens a document for a object"""
        assert not hasattr(obj, "_pane") or obj._pane is None
        # create document instance
        p = _class(self.docBook, self, obj)
        obj._pane = p
        self.docBook.AddPage(p, obj.name, True, obj.bitmap_index)

    def OnResetPerpective(self, event):
        """Reset default perspective"""
        self.m_mgr.LoadPerspective(self._mainPerspective)

    def OnOutputMessage(self, event):
        """Receives a output mensage for showing in output pane"""
        s = self.auxOutput.GetNumberOfLines() - 200
        if s > 0:
            k = sum((self.auxOutput.GetLineLength(i) for i in range(0, s))) + s
            self.auxOutput.Remove(0, k)
        self._output.LogTextAtLevel(event.level, event.message)

    def StopRunning(self):
        """Stop running process"""
        if not hasattr(self, '_runProcess'):
            # there are now a running process!!
            return True
        if self._runProcess:
            os.killpg(os.getpgid(self._runProcess.pid), signal.SIGTERM)

    def RunPythonScript(self, script):
        """Executes a python script, redirecting output to the output"""
        import subprocess
        import threading

        if hasattr(self, '_runProcess'):
            # there are now a running process!!
            return False

        # show output aux pane
        self.CleanOutputPane()
        if self.m_aux_panes.GetCurrentPage() != self.m_aux_output_pane:
            # we attempt to show output window if it exist (may be closed by the user)
            try:
                i = max(i for i in range(0, self.m_aux_panes.GetPageCount())
                    if self.m_aux_panes.GetPage(i) == self.m_aux_output_pane)
                self.m_aux_panes.SetSelection(i)
            except:
                pass

        def worker_pipe(self):
            """funcion que realiza el trabajo en el thread"""
            handler = self.GetEventHandler()
            handler.AddPendingEvent(wxx.LoggerEvent('watching process ...'))
            for line in iter(self._runProcess.stdout.readline, ""):
                handler.AddPendingEvent(wxx.LoggerEvent(line.strip()))
                wx.YieldIfNeeded()
            self._runProcess.stdout.close()
            self._runProcess.wait()
            del self._runProcess
            handler.AddPendingEvent(wxx.LoggerEvent('execution ended'))

        command = '/usr/bin/python {script}'.format(script=script)
        try:
            self._runProcess = subprocess.Popen(command,
                stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True,
                preexec_fn=os.setsid)
            threading.Thread(target=worker_pipe, args=(self,)).start()
        except:
            handler = self.GetEventHandler()
            handler.AddPendingEvent(wxx.LoggerEvent('execution failed'))

    def CleanOutputPane(self):
        """Clean the output pane"""
        self.auxOutput.SetValue(wx.EmptyString)

    def search_proc(self, text):
        """Search a the context specified files for text"""
        #activate output pane
        i = max(i for i in range(0, self.m_aux_panes.GetPageCount())
        if self.m_aux_panes.GetPage(i) == self.m_searchPane)
        self.m_aux_panes.SetSelection(i)
        #
        theApp = context.app
        maxResults = 200
        handler = self.GetEventHandler()
        for wrk in theApp.workspaces:
            for prj in wrk(model.Project):
                rdir = os.path.realpath(prj.dir)
                if not os.access(rdir, os.R_OK):
                    continue
                for dirName, subdirList, fileList in os.walk(rdir):
                    for fname in fileList:
                        #avoid scan hidden files
                        if re.match(r'^\.(.)*', fname):
                            continue
                        f = os.path.realpath(
                            os.path.join(dirName, fname))
                        if not os.access(f, os.R_OK):
                            continue
                        with open(f, "r") as sf:
                            cnt = 0
                            print f
                            for line in sf:
                                cnt = cnt + 1
                                try:
                                    if text in line:
                                        #post message(file/line)
                                        maxResults = maxResults - 1
                                        if maxResults <= 0:
                                            # avoid crashing
                                            handler.AddPendingEvent(wxx.SearchEvent(cnt, '**TRUNCATED**'))
                                            return 0
                                        handler.AddPendingEvent(
                                            wxx.SearchEvent(cnt, f))
                                except:
                                    pass
        return 0

    def OnFindInFiles(self, event):
        """Lanza el cuadro de dialogo de buscar"""
        theDialog = app.ui.dlg.FindInFilesDialog(self)
        if theDialog.ShowModal() != wx.ID_OK:
            return
        self.m_searchList.ClearAll()
        w, h = self.m_searchList.GetSize()
        self.m_searchList.InsertColumn(0, "file", width=w - 40)
        self.m_searchList.InsertColumn(1, "line", width=40)
        import threading
        threading.Thread(target=mainWindow.search_proc,
            args=(self, theDialog.searchString)).start()

    def OnSearchResults(self, event):
        """Put results in search pane"""
        m_list = self.m_searchList
        m_list.Append((event.fname, event.line))

    def OnSelectSearchItem(self, event):
        """Attempts to open the file"""
        index = event.GetIndex()
        try:
            import activity.files.ui.pane as fpane
            import model.file
            path = self.m_searchList.GetItem(index, 0).GetText()
            line = int(self.m_searchList.GetItem(index, 1).GetText())
            p = os.path.abspath(path)
            if re.match(r"(.)*\.cpp$", p) or re.match(r"(.)*\.h$", p):
                cls = fpane.FilePane
            elif re.match(r"(.)*\.py$", p):
                cls = fpane.FilePythonPane
            else:
                cls = fpane.TextPane
            for wrk in context.app.workspaces:
                items = wrk(model.file.File, filter=lambda x: x._file == p)
                if not len(items):
                    continue
                # Ordenamos la apertura del fichero
                obj = items[0]
                if hasattr(obj, '_pane') and obj._pane is not None:
                    self.docBook.SetSelection(
                        self.docBook.GetPageIndex(obj._pane))
                else:
                    self.OpenDoc(cls, obj)
                obj._pane.GotoLine(line - 1, True)
                self.m_searchList.SetFocus()
                break
        except:
            pass




import re
import copy
import wx
import wxx
import model
from activity.models.ui import ui as ui
import app.resources as rc


# Implementing NewMethod
class MemberMethodDialog(ui.NewMemberMethod):
    """
    This dialog allows to setup the attributes
    of a member function of struct or class.
    You can configure the method for not write
    his declaration and/or his implementation.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialization"""
        super(MemberMethodDialog, self).__init__(parent)
        self.container = container

        #new code for types
        self._types = dict([x._name, x] for x in container.types)
        self.m_choice1.AppendItems([x for x in self._types.keys() if x != '@'])

        self._template_types = []
        # we need to add types from template nested classes
        self._nested_template_types = container.template_types
        if len(self._nested_template_types) > 0:
            self.m_choice1.AppendItems(self._nested_template_types)
        self.m_choice1.SetSelection(self.m_choice1.FindString(''))
        self.choiceStr = ""
        self.m_comboBox5.SetFocus()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("method"))
        self.SetIcon(icon)

    def OnToggleTemplate(self, event):
        """Handles toggle event"""
        if self.m_checkBox63.GetValue():
            self.m_stStartTemplate.Enable(True)
            self.m_template.Enable(True)
            self.m_stEndTemplate.Enable(True)
            # templates cannot be virtual
            self.m_checkBox11.SetValue(False)
            self.m_checkBox11.Enable(False)
            self.m_checkBox21.SetValue(False)
            self.m_checkBox21.Enable(False)
        else:
            self.m_stStartTemplate.Enable(False)
            self.m_template.SetValue('')
            self.m_template.Enable(False)
            self.m_stEndTemplate.Enable(False)
            # non-templates can be virtual
            self.m_checkBox11.Enable(True)
            self.m_checkBox21.Enable(True)
        event.Skip()

    def removeTemplateTypes(self):
        """Remove template generated types from combo. This method only removes
        the in-place template types created by function template declaration."""
        # get current value
        iSel = self.m_choice1.GetCurrentSelection()
        if iSel != wx.NOT_FOUND:
            s = self.m_choice1.GetString(iSel)
        else:
            s = None
        for x in self._template_types:
            i = self.m_choice1.FindString(x)
            if i != wx.NOT_FOUND:
                self.m_choice1.Delete(i)
        if s is not None:
            self.m_choice1.SetStringSelection(s)

    def OnChangeTemplateSpecification(self, event):
        """Handles change text event"""
        # When the template specification changes, virtual types must be added
        text = self.m_template.GetValue()
        l = text.split(',')
        prologs = ['class', 'typename']
        r = []
        for u in l:
            v = u.split('=')[0].strip()
            for p in prologs:
                if v.find(p) == 0:
                    v = v.replace(p, '').strip()
                    if len(v) and v not in self._nested_template_types:
                        r.append(v)
                    break
        #remove previous template types
        self.removeTemplateTypes()
        # Append templates to available types
        if len(r):
            self.m_choice1.AppendItems(r)
            self._template_types = r
        event.Skip()

    def OnTypeChanged(self, event):
        """This event happens when the return type is changed. The main goal
        of this callback is handling template types for argument specification"""
        iSel = self.m_choice1.GetCurrentSelection()
        _type = self._types.get(self.m_choice1.GetString(iSel), None)
        template_args = False
        if _type is not None:
            if _type._template is not None:
                template_args = True
        if template_args is True:
            self.m_staticText65.Enable(True)
            self.m_template_args.Enable(True)
            self.m_staticText66.Enable(True)
        else:
            self.m_staticText65.Enable(False)
            self.m_template_args.Enable(False)
            self.m_staticText66.Enable(False)
            self.m_template_args.SetValue('')
        event.Skip()

    def Validate(self):
        """Validation"""
        self._name = self.m_comboBox5.GetValue()
        if len(self._name) == 0:
            wx.MessageBox("Method name must not be empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if re.match("^([A-Za-z_][0-9A-Za-z_]*|operator (=|==|!=|<|>|<=|>=|\+|-|\*|/|%|\+=|-=|\*=|/=|%=|!|&&|\|\||\+\+|--|&|\||\^|~|&=|\|=|\^=|<<|>>|<<=|>>=|->|\(\)|\[\]|new|delete|bool|int))$", self._name) is None:
            wx.MessageBox("Method name contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        iSel = self.m_choice1.GetCurrentSelection()
        if iSel == wx.NOT_FOUND:
            wx.MessageBox("Invalid type", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        typename = self.m_choice1.GetString(iSel)
        if self.m_checkBox63.IsChecked():
            self._template = self.m_template.GetValue()
        else:
            self._template = None
        #filter template types
        template_types = self._template_types + self._nested_template_types
        if typename in template_types:
            self._typei = model.cc.typeinst(
                type=self._types['@'],
                type_alias=typename,
                const=self.m_checkBox1.IsChecked(),
                ptr=self.m_checkBox3.IsChecked(),
                ref=self.m_checkBox2.IsChecked(),
                ptrptr=self.m_checkBox4.IsChecked(),
                constptr=self.m_checkBox5.IsChecked(),
                )
        else:
            _type = self._types[typename]
            if _type._template is not None:
                #we construct type instance with explicit arguments
                type_args = self.m_template_args.GetValue()
                self._typei = model.cc.typeinst(
                    type=_type,
                    const=self.m_checkBox1.IsChecked(),
                    ptr=self.m_checkBox3.IsChecked(),
                    ref=self.m_checkBox2.IsChecked(),
                    ptrptr=self.m_checkBox4.IsChecked(),
                    constptr=self.m_checkBox5.IsChecked(),
                    type_args=type_args
                    )
            else:
                self._typei = model.cc.typeinst(
                    type=_type,
                    const=self.m_checkBox1.IsChecked(),
                    ptr=self.m_checkBox3.IsChecked(),
                    ref=self.m_checkBox2.IsChecked(),
                    ptrptr=self.m_checkBox4.IsChecked(),
                    constptr=self.m_checkBox5.IsChecked(),
                    )
        iSel = self.m_choice2.GetCurrentSelection()
        self._access = self.m_choice2.GetString(iSel)
        self._static = self.m_checkBox6.IsChecked()
        self._virtual = self.m_checkBox11.IsChecked()
        self._pure = self.m_checkBox21.IsChecked()
        self._inline = self.m_checkBox31.IsChecked()
        self._const_method = self.m_checkBox41.IsChecked()
        self._note = self.m_richText1.GetValue()
        self._declare = self.m_checkBox85.IsChecked()
        self._implement = self.m_checkBox86.IsChecked()
        return True

    def get_kwargs(self):
        """Returns kwargs dictionary suitable for object creation"""
        kwargs = {}
        kwargs['parent'] = self.container
        kwargs['name'] = self._name
        kwargs['type'] = self._typei
        kwargs['access'] = self._access
        kwargs['static'] = self._static
        kwargs['virtual'] = self._virtual
        kwargs['pure'] = self._pure
        kwargs['inline'] = self._inline
        kwargs['template'] = self._template
        kwargs['template_types'] = self._template_types
        kwargs['note'] = self._note
        kwargs['declare'] = self._declare
        kwargs['implement'] = self._implement
        return kwargs

    def CopyAttributes(self, method):
        """Copy attributes to method"""
        method._name = self._name
        method._typei = copy.copy(self._typei)
        method._access = self._access
        method._static = self._static
        method._virtual = self._virtual
        method._pure = self._pure
        method._inline = self._inline
        method._const_method = self._const_method
        method.note = self._note
        method._template = self._template
        method._template_types = self._template_types
        method._declare = self._declare
        method._implement = self._implement

    def SetAttributes(self, method):
        """Setup attributes for editing already method"""
        self.m_comboBox5.SetValue(method._name)
        iSel = self.m_choice2.FindString(method._access)
        self.m_choice2.SetSelection(iSel)
        self.m_checkBox6.SetValue(method._static)
        self.m_checkBox11.SetValue(method._virtual)
        self.m_checkBox21.SetValue(method._pure)
        self.m_checkBox31.SetValue(method._inline)
        self.m_checkBox41.SetValue(method._const_method)
        self.m_richText1.SetValue(method._note)
        self.SetTitle("Edit method")
        if method._template is not None:
            self.m_checkBox63.SetValue(True)
            self.m_stStartTemplate.Enable(True)
            self.m_template.Enable(True)
            self.m_stEndTemplate.Enable(True)
            self.m_template.SetValue(method._template)
            # if template is specified and the method holds arguments
            # using that types, the template specification cannot be changed
            for arg in method[model.cc.Argument]:
                if arg._typei._type_alias in method._template_types:
                    self.m_checkBox63.Enable(False)
                    self.m_checkBox63.SetToolTipString(
                        'Template cannot be edited because arguments using template types exist.')
                    self.m_stStartTemplate.Enable(False)
                    self.m_template.Enable(False)
                    self.m_stEndTemplate.Enable(False)
                    break
        ti = method._typei
        iSel = self.m_choice1.FindString(ti.type_name)
        self.m_choice1.SetSelection(iSel)
        self.m_checkBox1.SetValue(ti._const)
        self.m_checkBox3.SetValue(ti._ptr)
        self.m_checkBox2.SetValue(ti._ref)
        self.m_checkBox4.SetValue(ti._ptr_to_ptr)
        self.m_checkBox5.SetValue(ti._const_ptr)
        if ti._type_args is not None:
            self.m_staticText65.Enable(True)
            self.m_template_args.Enable(True)
            self.m_staticText66.Enable(True)
            self.m_template_args.SetValue(ti._type_args)
        self.m_checkBox85.SetValue(method._declare)
        self.m_checkBox86.SetValue(method._implement)

    def OnPointerToggle(self, event):
        """ptr toggle gui"""
        if self.m_checkBox3.IsChecked():
            self.m_checkBox4.Enable(True)
            self.m_checkBox5.Enable(True)
        else:
            self.m_checkBox4.SetValue(False)
            self.m_checkBox5.SetValue(False)
            self.m_checkBox4.Enable(False)
            self.m_checkBox5.Enable(False)
        event.Skip()

    def OnKeyDown(self, event):
        """Listbox selection"""
        keycode = event.GetKeyCode()
        if keycode == wx.WXK_UP or keycode == wx.WXK_NUMPAD_UP:
            i = self.m_choice1.GetSelection()
            if i is not wx.NOT_FOUND and i > 0:
                self.m_choice1.SetSelection(i - 1)
        elif keycode == wx.WXK_DOWN or keycode == wx.WXK_NUMPAD_DOWN:
            i = self.m_choice1.GetSelection() + 1
            if i > wx.NOT_FOUND and i < len(self._types):
                self.m_choice1.SetSelection(i)
        elif keycode < 256:
            keychar = chr(keycode)
            if keychar.isalnum() or keycode is wx.WXK_SPACE:
                self.choiceStr += keychar.lower()
                for t in self._types:
                    tl = t.lower()
                    if tl.find(self.choiceStr) == 0:
                        sel = self.m_choice1.FindString(t)
                        if sel is not wx.NOT_FOUND:
                            self.m_choice1.SetSelection(sel)
                            if keycode is not wx.WXK_SPACE:
                                event.Skip()
                            return
            self.choiceStr = ""
        event.Skip()

    # Handlers for NewMethod events.
    def OnCancel(self, event):
        """cancel event handler"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """ok event handler"""
        if self.Validate():
            self.EndModal(wx.ID_OK)





"""Subclass of FilePane, which is generated by wxFormBuilder."""

import wx
import wxx
import wx.stc
from ctx import theContext as context
import tran
import tran.TransactionStack as stack
from app.ui.pane import BasePane
from activity.files.ui import ui as ui
from activity.models.ui import ui as amui

CLOSE_PANE_ID = wx.ID_HIGHEST + 100


# Implementing FilePane
class FilePythonPane(BasePane, ui.FilePythonPane):
    """Implements code editor"""
    def __init__(self, parent, mainframe, fileobj):
        """Intialization of method editor"""
        super(FilePythonPane, self).__init__(parent)
        self._parent = parent
        self._mainframe = mainframe
        self._object = fileobj
        self._types = {}
        self._defaulBkColor = self.GetBackgroundColour()
        self.m_editor.SetText(fileobj.GetText())
        self.m_editor.HandleBreakpoints(fileobj.breakpoints)
        self.m_editor.HandleBookmarks(fileobj.bookmarks)
        self.m_editor.EnableBreakpoints(True)
        self.m_editor.EnableBookmarks(True)
        self.m_editor.Initialize(self._types)
        self.m_editor.SetReadOnly(fileobj._readOnly)
        # setup mark zone for python
        self.m_editor.SetEdgeMode(wx.stc.STC_EDGE_BACKGROUND)
        self.m_editor.SetEdgeColumn(80)
        self.Layout()

        wx.stc.EVT_STC_CHANGE(self.m_editor, self.m_editor.GetId(),
            self.OnEditorChange)

        frame = context.frame
        self.BindSpecial(wx.EVT_MENU, handler=frame.CloseCurrentDocPane, id=CLOSE_PANE_ID)
        self.BindSpecial(wx.EVT_MENU, self.OnDelete, id=frame.delete.GetId())
        self.BindSpecial(wx.EVT_MENU, self.OnSave, id=frame.saveProject.GetId())
        self.BindSpecial(wx.wx.EVT_UPDATE_UI, self.OnUpdateSave, id=frame.saveProject.GetId())

        aTable = wx.AcceleratorTable([
            wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_DELETE, frame.delete.GetId()),
            wx.AcceleratorEntry(wx.ACCEL_ALT | wx.ACCEL_CTRL, ord('F'), amui.ID_TOGGLE_FULLSCREEN),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, ord('W'), CLOSE_PANE_ID)
        ])
        self.SetAcceleratorTable(aTable)

    @property
    def breakpoint(self):
        """return info about if the current line holds breakpoint"""
        return self.m_editor.breakpoint

    def ToggleBreakpoint(self, event):
        """toggle breakpoint"""
        self.m_editor.ToggleBreakpoint(event)

    @property
    def line(self):
        """return current line"""
        return self.m_editor.GetCurrentLine()

    def SetReadOnly(self, value=True):
        """Set readonly flag. Used for debug"""
        self.m_editor.SetReadOnly(value)

    def RestoreReadOnly(self):
        """Set readonly flag"""
        self.m_editor.SetReadOnly(self._object._readOnly)

    def Refresh(self):
        """Update editor from external changes (like undo/redo)"""
        oldText = self.m_editor.GetText()
        newText = self._object.GetText()
        if oldText == newText:
            return
        line = self.m_editor.GetFirstVisibleLine()
        pos = self.m_editor.GetCurrentPos()
        self.m_editor.Freeze()
        self.m_editor.SetText(newText)
        self.m_editor.SetModified(False)
        try:
            self.m_editor.SetFirstVisibleLine(line)
            self.m_editor.SetCurrentPos(pos)
        except:
            pass
        self.m_editor.Thaw()

    def OnEditorChange(self, event):
        """Called when editor status change"""
        if self.m_editor.GetModify():
            self.SetBackgroundColour(wx.LIGHT_GREY)
        else:
            self.SetBackgroundColour(self._defaulBkColor)

    def GotoLine(self, line, select=False):
        """Called for goto to line"""
        self.m_editor.GotoLine(line, select)

    def DebugLine(self, line):
        """"""
        self.m_editor.GotoLine(line, False)
        start = self.m_editor.PositionFromLine(line)
        length = self.m_editor.LineLength(line)
        self.m_editor.StartStyling(start, 32)
        self.m_editor.SetStyling(length, 32)
        if hasattr(self, '_debugLine'):
            if self._debugLine != line:
                self.m_editor.GotoLine(self._debugLine, False)
                start = self.m_editor.PositionFromLine(self._debugLine)
                length = self.m_editor.LineLength(self._debugLine)
                self.m_editor.StartStyling(start, 32)
                self.m_editor.SetStyling(length, 0)
        self._debugLine = line

    def Select(self, sline, scol, eline, ecol):
        """Called for select range"""
        self.m_editor.Select(sline, scol, eline, ecol)

    @tran.TransactionalMethod()
    def Commit(self):
        """Do commit changes"""
        if self.m_editor.GetModify():
            self._object.SaveState()
            self._object.SetText(self.m_editor.GetText())
            self.m_editor.SetModified(False)
            self.SetBackgroundColour(self._defaulBkColor)
            stack.instance.SetName('modify file {}'.format(self._object._name))
            return True
        else:
            return False

    def OnDelete(self, event):
        """Handle delete key"""
        (b, e) = self.m_editor.GetSelection()
        if b == e:
            self.m_editor.CharRight()
        self.m_editor.DeleteBack()

    def OnUpdateSave(self, event):
        """Handles update event"""
        if not self.is_event_handler:
            print "Called on inactive window"
            event.Skip()
            return
        if self.FindFocus() == self.m_editor:
            event.Enable(self.m_editor.GetModify())
        else:
            event.Skip()

    def OnSave(self, event):
        """Handles save event"""
        if self.FindFocus() == self.m_editor:
            self.Commit()
        else:
            event.Skip()





# -*- coding: utf-8 -*-
"""
Created on Sun Dec 22 22:31:28 2013

@author: mel
"""

import model
import tran
import app.resources as rc


class Argument(model.TComponent):
    """Implements argument representation"""

    context_container = True

    #visual methods
    @tran.TransactionalMethod('move argument {}')
    def drop(self, to):
        """drop this elemento to another place"""
        target = to.inner_argument_container
        if not target or to.project != self.project:
            return False  # avoid move arguments between projects
        index = 0  # trick for insert as first child
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        """Initialization"""
        # arguments:
        # ... default : the default or the call value
        # ... context : the use case. may be:
        #    ... declare : when the argument represents a argument declaration
        #    ....value : when the argument represents a value in call (the name is irrelevant)
        #    ....keyword : when the argument represent a keyword value in call (default is mandatory)
        #    ....starargs : when the argument represent a star args in call
        #    ....kwargs : when the argument represent a keyword args in call
        #
        # normal context is declare. The rest of contexts are argument-reuse for few cases, like
        # decorator instantiations
        self._default = kwargs.get('default', '')
        self._context = kwargs.get('context', 'declare')
        super(Argument, self).__init__(**kwargs)
        assert(self._context != 'keyword' or self._default)
        container = self.outer_class or self.outer_module
        container._lastSrcTime = None
        container._lastHdrTime = None
        k = self.inner_module or self.inner_package
        if k:
            k.ExportPythonCodeFiles()

    def Delete(self):
        """Handle delete"""
        k = self.inner_module or self.inner_package
        super(Argument, self).Delete()
        if k:
            k.ExportPythonCodeFiles()

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['default'] = self._default
        kwargs.update(super(Argument, self).get_kwargs())
        return kwargs

    def OnUndoRedoChanged(self):
        """Update from app"""
        self.parent.OnUndoRedoChanged()
        super(Argument, self).OnUndoRedoChanged()

    def OnUndoRedoRemoving(self, root=True):
        """Do required actions for removing"""
        super(Argument, self).OnUndoRedoRemoving(root)
        self.parent.OnUndoRedoChanged()

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        self.parent.OnUndoRedoChanged()
        super(Argument, self).OnUndoRedoAdd()

    @property
    def label(self):
        """Get tree label"""
        if self._context == 'declare':
            if self._default:
                return '{self._name}={self._default}'.format(self=self)
            else:
                return self._name
        if self._context == 'value':
            return self._name
        if self._context == 'keyword':
            return '{self._name}={self._default}'.format(self=self)
        if self._context == 'starargs':
            return '*{self._default}'.format(self=self)
        if self._context == 'kwargs':
            return '**{self._default}'.format(self=self)

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex('py_argument')




# -*- coding: utf-8 -*-

# This file defines a ver basic private clipboard
# whose mission is to handle data and provide info about

from deco import classproperty
import uuid


class clipboard(object):
    "Basic clipboard definition"
    # propiedades
    __registered__classes = {}
    __current_data = None

    def __init__(self):
        """Initialization. Setup handlers"""
        super(clipboard, self).__init__()

    @classproperty
    def is_empty(cls):
        """has data?"""
        return not cls.__current_data

    @classmethod
    def register(cls, type):
        """register a new class or access it"""
        if type not in cls.__registered__classes:
            cls.__registered__classes[type] = uuid.uuid4()
        return cls.__registered__classes[type]

    @classmethod
    def copy(cls, data):
        """copy data to clipboard"""
        assert type(data) in cls.__registered__classes
        if id(data) != id(cls.__current_data):
            cls.__current_data = data

    @classproperty
    def info(cls):
        """return clipboard info"""
        return cls.__current_data and cls.__registered__classes[
            type(cls.__current_data)]

    @classproperty
    def data(cls):
        """return the current data"""
        return cls.__current_data




# -*- coding: utf-8 -*-

import os
import xml.etree.ElementTree as et


def relationMembers(**kwargs):
    """
    Este metodo lee la definicion de los metodos de un
    lado de la relacion.

    Parametros:
        version: version de metodos por defecto, standard
        part: lado de la relacion (to o from)
        filter: dimodelionario de booleanos de selemodelion de version
        dictionary: dimodelionario de valores a utilizar en la evaluacion

    Retorna:

        None en caso de error
        Array de definicion de metodos.
    """
    doc = '{}Members.xml'.format(kwargs['part'])
    path = os.path.join(os.getcwd(), 'plugin', 'models', 'relation',
        kwargs.get('version', 'standard'), doc)
    key = kwargs.get('filter', None)
    if not os.path.isfile(path):
        print('{} is missing'.format(path))
        return None
    try:
        tree = et.parse(path)  # retorna et.ElementTree
        root = tree.getroot()  # retorna et.Element
        dictionary = kwargs.get('dictionary', {})
        defs = root.find('definitions')
        if defs is not None:
            for d in defs:
                text = d.findtext('.')
                for x, y in [text.split('=')]:
                    v = y.format(**dictionary)
                    dictionary[x] = v
        members = root.find('members')
        if members is None:
            print('missing members')
            return None
        result = []
        for member in members:
            keys = {}
            for x in member.keys():
                keys[x] = member.get(x).format(**dictionary)
                if x == 'type':
                    keys[x] = eval(keys[x], dictionary)
            result.append(keys)
        return result
    except Exception, e:
        print('Exception {msg} failed reading {p}'.format(
            msg=e.message, p=path))
        return None


def relationMethods(**kwargs):
    """
    Este metodo lee la definicion de los metodos de un
    lado de la relacion.

    Parametros:
        version: version de metodos por defecto, standard
        part: lado de la relacion (to o from)
        filter: dimodelionario de booleanos de selemodelion de version
        dictionary: dimodelionario de valores a utilizar en la evaluacion

    Retorna:

        None en caso de error
        Array de definicion de metodos.

    Comentarios:

        Los parametros de filtro permiten seleccionar entre
        diferentes versiones por ejemplo, de compatibilidad C++
        o niveles de seguridad (criticos), implementacion estatica
        presencia de metodos de busqueda, etc. Se selemodeliona
        el mejor matching y se excluyen aquellos metodos que
        no tengan ninguna version compatible con los filtros.

        Respecto de los simbolos utilizados en las declaraciones,
        esto es un aspecto pendiente de normalizar y documentar,
        ya que por ahora algunos de ellos dependen de la implementacion
        de los valores miembro.


    """
    doc = '{}Methods.xml'.format(kwargs['part'])
    path = os.path.join(os.getcwd(), 'plugin', 'models', 'relation',
        kwargs.get('version', 'standard'), doc)
    key = kwargs.get('filter', None)
    if not os.path.isfile(path):
        print('{} is missing'.format(path))
        return None
    try:
        tree = et.parse(path)
        root = tree.getroot()
        dictionary = kwargs.get('dictionary', {})
        # read definitions and add it to dict
        defs = root.find('definitions')
        if defs is not None:
            for d in defs:
                text = d.findtext('.')
                for x, y in [text.split('=')]:
                    v = y.format(**dictionary)
                    dictionary[x] = v

        methods = root.find('methods')
        if methods is None:
            print('missing methods')
            return None
        result = []
        #elements = methods.getchildren()
        #for method in elements:
        for method in methods:
            elem = {}
            keys = {}
            for x in method.keys():
                keys[x] = method.get(x).format(**dictionary)
                if x == 'type':
                    keys[x] = eval(keys[x], dictionary)
            elem['key'] = keys
            # tomamos los argumentos
            args = method.find('args')
            if args is None:
                print('missing args')
                return None
            arglist = []
            for arg in args:
                akeys = {}
                for x in arg.keys():
                    akeys[x] = arg.get(x).format(**dictionary)
                    if x == 'type':
                        akeys[x] = eval(akeys[x], dictionary)
                arglist.append(akeys)
            elem['arglist'] = arglist
            # tomamos los posibles cuerpos
            contents = method.findall('content')
            if contents is None:
                elem['decl'] = ''
                continue
            best_match = -1
            best_content = None
            for content in contents:
                #verificamos el filtro
                f = content.get('filter', None)
                ok = True
                match = 0
                if f is not None:
                    f = f.strip()[1:-1]  # quitamos los { y }
                    fk = dict([(x.strip(), (y.strip() == 'True'))
                        for x, y in [list(v.split(':')) for v
                        in f.split(',')]])
                    # para pasar el filtro, deben coincidir los valores
                    for k in key:
                        if k in fk and key[k] != fk[k]:
                            ok = False
                            break
                        match += 1
                if not ok:
                    continue
                if match > best_match:
                    best_content = content
                    best_match = match
            # si no se pasa el filtro es un error
            if best_content is None:
                continue
            elem['decl'] = best_content.findtext('.').format(**dictionary)
            result.append(elem)
        return result
    except Exception, e:
        print('Exception {msg} failed reading {p}'.format(
            msg=e.message, p=path))
        return None


def relationClasses(**kwargs):
    """
    Realiza la construccion de las clases anidadas
    definidas por la relacion
    """
    import model
    doc = '{}Classes.xml'.format(kwargs['part'])
    path = os.path.join(os.getcwd(), 'plugin', 'models', 'relation',
        kwargs.get('version', 'standard'), doc)
    #key = kwargs.get('filter', None)
    if not os.path.isfile(path):
        print('{} is missing'.format(path))
        return
    try:
        tree = et.parse(path)
        root = tree.getroot()
        dictionary = kwargs.get('dictionary', {})
        parent = dictionary.get('parent', None)
        if not parent:
            print('parent is missing')
            return
        # read definitions and add it to dict
        defs = root.find('definitions')
        if defs is not None:
            for d in defs:
                text = d.findtext('.')
                for x, y in [text.split('=')]:
                    v = y.format(**dictionary)
                    dictionary[x] = v

        classes = root.find('classes')
        if classes is None:
            print('missing classes')
            return
        for _class in classes:
            keys = {}
            # leemos los atributos de la clase
            for x in _class.keys():
                keys[x] = _class.get(x).format(**dictionary)
            # ponemos atributos
            keys.update({'parent': parent, 'readonly': kwargs.get('readonly', True)})
            # creamos la clase
            cls = model.cc.Class(**keys)
            dictionary.update({cls.name: cls})
            # buscamos los tipos definidos en la clase
            types = _class.find('types')
            for type in types:
                keys = {}
                for x in type.keys():
                    keys[x] = type.get(x).format(**dictionary)
                keys.update({'parent': cls})
                t = model.cc.Type(**keys)
                dictionary.update({t.name: t})
            # buscamos los miembros
            members = _class.find("members")
            for member in members:
                keys = {}
                for x in member.keys():
                    keys[x] = member.get(x).format(**dictionary)
                    if x in ['type', 'static', 'const', 'volatile', 'virtual', 'ptr', 'ref', 'mutable']:
                        keys[x] = eval(keys[x], dictionary)
                keys['type'] = model.cc.typeinst(**keys)
                keys.update({'parent': cls})
                model.cc.MemberData(**keys)
            # constructores
            ctors = _class.find('ctors')
            for _ctor in ctors:
                keys = {'parent': cls, 'name': cls.name, 'autoargs': False}
                for x in _ctor.keys():
                    keys[x] = _ctor.get(x).format(**dictionary)
                    if x == 'inline':
                        keys[x] = eval(keys[x], dictionary)
                ctor = model.cc.Constructor(**keys)
                # argumentos
                args = _ctor.find('args')
                for arg in args:
                    keys = {'parent': ctor}
                    for x in arg.keys():
                        keys[x] = arg.get(x).format(**dictionary)
                        if x == 'type':
                            keys[x] = eval(keys[x], dictionary)
                    keys['type'] = model.cc.typeinst(**keys)
                    model.cc.Argument(**keys)

            # buscamos los metodos
            methods = _class.find('methods')
            for _method in methods:
                keys = {'parent': cls}
                for x in _method.keys():
                    k = keys[x] = _method.get(x).format(**dictionary)
                    if x == 'type':
                        if k:
                            keys[x] = eval(k, dictionary)
                        else:
                            keys[x] = dictionary[k]  # implicit type
                    elif x in ['static', 'const', 'volatile', 'virtual', 'ptr', 'ref', 'inline']:
                        keys[x] = eval(k, dictionary)
                keys['type'] = model.cc.typeinst(**keys)
                keys.update({'parent': cls})

                method = model.cc.MemberMethod(**keys)
                # argumentos
                args = _method.find('args')
                for arg in args:
                    keys = {}
                    for x in arg.keys():
                        keys[x] = arg.get(x).format(**dictionary)
                        if x in ['type', 'static', 'const', 'volatile', 'ptr', 'ref']:
                            keys[x] = eval(keys[x], dictionary)
                    keys['type'] = model.cc.typeinst(**keys)
                    keys.update({'parent': method})
                    model.cc.Argument(**keys)
            #destructor
            _dtor = _class.find('dtor')
            if _dtor:
                keys = {}
                for x in _dtor.keys():
                    keys[x] = _dtor.get(x).format(**dictionary)
                    if x in ['virtual', 'inline']:
                        keys[x] = eval(keys[x], dictionary)
                keys.update({'parent': cls, 'name': cls.name})
                model.cc.Destructor(**keys)

    except Exception, e:
        print('Exception {msg} failed reading {p}'.format(
            msg=e.message, p=path))
        return



# -*- coding: utf-8 -*-

# -*- coding: utf-8 -*-

"""
Created on Sun Dec 15 19:22:32 2013

@author: mel
"""
import wx
import model
import os
import os.path
import app.resources as rc


class GitFile(model.TComponent):
    """Implements c++ class representation"""
    folder_container = True

    def __init__(self, **kwargs):
        """Initialization method"""
        self._lastHdrTime = None
        self._lastSrcTime = None
        self._name = kwargs['name']
        self._parent = kwargs['parent']
        #Query the status: the file is modified, staged or deleted?
        self._status = self.repo.status(self.rpath)
        wx.YieldIfNeeded()
        super(GitFile, self).__init__(**kwargs)
        # iterate over the tree
        # assert len(self._data.hexsha) == 40

    @property
    def repo(self):
        """return the repo"""
        return self.parent.repo

    def update_status(self):
        """Update the file status"""
        wx.YieldIfNeeded()
        self._status = self.repo.status(self.rpath)
        self.OnUndoRedoChanged()

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {'data': self._data}
        kwargs.update(super(GitFile, self).get_kwargs())
        return kwargs

    @property
    def can_delete(self):
        """Check abot if class can be deleted"""
        return super(GitFile, self).can_delete

    def Delete(self):
        """Delete diagram objects"""
        super(GitFile, self).Delete()

    def RemoveRelations(self):
        """Utility for undo/redo"""
        super(GitFile, self).RemoveRelations()

    def RestoreRelations(self):
        """Utility for undo/redo"""
        super(GitFile, self).RestoreRelations()

    def SaveState(self):
        """Utility for saving state"""
        super(GitFile, self).SaveState()

    def OnUndoRedoRemoving(self, root=True):
        """Prepare object to delete"""
        super(GitFile, self).OnUndoRedoRemoving(root)

    def OnUndoRedoChanged(self):
        """Update from app"""
        #when the class is updated, ctor's and dtors, must be updated
        super(GitFile, self).OnUndoRedoChanged()

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(GitFile, self).OnUndoRedoAdd()

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex(self._status)

    @property
    def path(self):
        """Access for this element"""
        return os.path.join(self.parent.path, self._name)

    @property
    def rpath(self):
        """Access for this element"""
        return os.path.join(self.parent.rpath, self._name)

    @property
    def label(self):
        """Get tree label"""
        return '{self._name}'.format(self=self)



